#include "AnimationManager.h"
#include "AnimationObjectPool.h"
#include "Animation.h"
#include "Definitions.h"

AnimationManager::AnimationManager() {
	pPool = new AnimationObjectPool;
}

AnimationManager::~AnimationManager() {
	for (std::vector<Animation*>::iterator it = spriteList.begin(); it != spriteList.end(); ++it) {
		pPool->RecycleAnimation(*it);
	}
	spriteList.empty();
	delete pPool;
}

AnimationManager* AnimationManager::instance = nullptr;

void AnimationManager::privTerminate() {
	
	delete instance;
}

void AnimationManager::privRequestRegenerationAnimation(sf::Vector2f pos) {
	Animation* nAnim = this->privRequestAnimationHelper();
	//details about regen anim
	nAnim->Initialize(pos, AnimatedSprite(ResourceManager::GetTexture("spawnSheet"), 3, 2, 16, 0, 3, true, false), 2.0f, 2.0f);
	spriteList.push_back(nAnim);
}

void AnimationManager::privRequestDeathAnimation(sf::Vector2f pos) {
	Animation* nAnim = this->privRequestAnimationHelper();
	nAnim->Initialize(pos, AnimatedSprite(ResourceManager::GetTexture("deathSheet"), 4, 2, 16, 0, 7, true, false),1.0f, 1.0f);
	spriteList.push_back(nAnim);
}

void AnimationManager::privRequestSpiderDeathAnimation(sf::Vector2f pos, int dist) {
	Animation* nAnim = this->privRequestAnimationHelper();
	if (dist <= cn::SPIDER_DISTANCE_NEAR) {
		nAnim->Initialize(pos, AnimatedSprite(ResourceManager::GetTexture("FullSheet"), 14, 70, 1.0f, 34, 34, true, false), 2.0f, 2.0f);
	}
	else if (dist <= cn::SPIDER_DISTANCE_MED) {
		nAnim->Initialize(pos, AnimatedSprite(ResourceManager::GetTexture("FullSheet"), 14, 70, 1.0f, 20, 20, true, false), 2.0f, 2.0f);
	}
	else {
		nAnim->Initialize(pos, AnimatedSprite(ResourceManager::GetTexture("FullSheet"), 14, 70, 1.0f, 6, 6, true, false), 2.0f, 2.0f);
	}
	nAnim->SetAlarm(0, 1.0f);
	spriteList.push_back(nAnim);
 }

Animation* AnimationManager::privRequestAnimationHelper() {
	Animation* nAnim = pPool->RequestAnimation();
	nAnim->SetExternalManagement(RecycleAnimation);
	return nAnim;
}

void AnimationManager::privRecycleAnimation(GameObject* go) {
	Animation* id = static_cast<Animation*>(go);
	spriteList.erase(std::find(spriteList.begin(), spriteList.end(), id));
	pPool->RecycleAnimation(id);
}

void AnimationManager::privUpdate() {

}

