#ifndef ANIMATION_MANAGER_H
#define ANIMATION_MANAGER_H

#include "Game Components/TEAL/CommonElements.h"
#include <vector>

class Animation;
class AnimationObjectPool;

class AnimationManager
{
private:
	AnimationManager();
	AnimationManager(const AnimationManager&) = delete;
	AnimationManager& operator=(const AnimationManager&) = delete;
	~AnimationManager();
	static AnimationManager* instance;
	static AnimationManager& getInstance() {
		if (!instance)
			instance = new AnimationManager();
		return *instance;
	}
public:
	//public static functions

	static void RequestRegenerationAnimation(sf::Vector2f pos) { getInstance().privRequestRegenerationAnimation(pos); }
	static void RequestDeathAnimation(sf::Vector2f pos) { getInstance().privRequestDeathAnimation(pos); }
	static void RequestSpiderDeathAnimation(sf::Vector2f pos, int dist) { getInstance().privRequestSpiderDeathAnimation(pos, dist); }
	static void RecycleAnimation(GameObject* anim) { getInstance().privRecycleAnimation(anim); }
	static void Terminate() { getInstance().privTerminate(); }
	static void Update() { getInstance().privUpdate(); }
	
private:
	//private non-static functions
	void privRequestRegenerationAnimation(sf::Vector2f);
	void privRequestDeathAnimation(sf::Vector2f);
	void privRequestSpiderDeathAnimation(sf::Vector2f, int);
	Animation* privRequestAnimationHelper();
	void privTerminate();
	void privUpdate();
	void privRecycleAnimation(GameObject*);
private:
	//private members
	std::vector<Animation*> spriteList;
	AnimationObjectPool* pPool;
};

#endif
