#include "BulletManager.h"
#include "Bullet.h"
#include "BulletObjectPool.h"
#include "Definitions.h"
#include "ProfileManager.h"
#include "Profile.h"

BulletManager::BulletManager() {
	BulletCount = 0;
	BulletExcessRequest = false;
	pPool = new BulletObjectPool();
}

BulletManager* BulletManager::instance = nullptr;

BulletManager::~BulletManager() {
	delete this->pPool;
}

void BulletManager::privTerminate() {
	delete instance;
}

bool BulletManager::privIsAtMaxBullets() {
	return (this->BulletCount >= cn::MAX_BULLETS_ON_SCREEN);
}

Bullet* BulletManager::privRequestBullet() {
	if (this->BulletCount >= cn::MAX_BULLETS_ON_SCREEN) {
		BulletExcessRequest = true;
		return nullptr;
	}
	else {
		return this->GetBullet();
	}
	
}


Bullet* BulletManager::GetBullet() {
	this->BulletCount++;
	Bullet* tBullet = this->pPool->GetBullet();
	tBullet->SetExternalManagement(RecycleBullet);
	tBullet->Initialize();
	liveBullets.push_back(tBullet);
	return tBullet;
}

void BulletManager::privRecycleBullet(GameObject* go) {
	this->pPool->ReturnBullet(static_cast<Bullet*>(go));
	//static_cast<Bullet*>(go)->DeInitialize();
	std::vector<Bullet*>::iterator it = std::find(liveBullets.begin(), liveBullets.end(), static_cast<Bullet*>(go));
	liveBullets.erase(it);
	if (BulletExcessRequest) {
		BulletExcessRequest = false;
		ProfileManager::GetCurrentPlayer()->FillBulletRequest(this->GetBullet());
	}
	this->BulletCount--;

}

void BulletManager::privKillAllBullets() {
	std::vector<Bullet*>::iterator it;
	for (it = liveBullets.begin(); it != liveBullets.end(); ++it) {
		(*it)->MarkForDestroy();
	}
}