#include "CentipedeBody.h"
#include "MoveState.h"
#include "MoveState_Halt.h"
#include "Bullet.h"
#include "Centipede.h"
#include "ScoreManager.h"
#include "ScoreCommand.h"
#include "SoundManager.h"
#include "AnimationManager.h"

CentipedeBody::CentipedeBody() {
	this->bitmap = &ResourceManager::GetTextureBitmap("centipedeBodySprite");
	this->cSprite = AnimatedSprite(ResourceManager::GetTexture("centipedeBodySprite"), 8, 2, 8, 0, 7, true, true);
	//this->Initialize();
	this->cSprite.setScale(2.0f, 2.0f);
	this->frameCounter = 0;

	this->pDeath = ScoreManager::GetScoreCommand(ScoreManager::SCORE_EVENTS::CentipedeBodyDeath);

	this->SetCollider(this->cSprite, *bitmap);
}

void CentipedeBody::Initialize(Centipede* pCent, int spd, int gpos) {
	CentipedeSegment::Initialize(pCent, spd, gpos);
	RegisterCollision<CentipedeBody>(*this);
	this->gridPos.x = gpos;
	this->gridPos.y = -1;
	frameCounter = 0;
	this->cPos = sf::Vector2f(gridPos.x * cn::GRID_SEGMENT_WIDTH, gridPos.y * cn::GRID_SEGMENT_HEIGHT);
	this->pMoveState = &MoveFSM::HALT;
}

void CentipedeBody::DeInitialize() {
	this->gridPos.x = 3;
	this->gridPos.y = 3;
 	this->pMoveState = nullptr;
	this->cPos.x = 33;
	this->cPos.y = 33;
	this->pCentipede = nullptr;
	this->speed = 10;
	this->frameCounter = 1;
}

void CentipedeBody::Update() {
	this->cPos.x += this->pMoveState->getMoveOffsets(frameCounter, this->speed).x;
	this->cPos.y += this->pMoveState->getMoveOffsets(frameCounter, this->speed).y;
	//Trace::out("Centipede BODY Pos: (%f, %f)\n", this->cPos.x, this->cPos.y);
	frameCounter++;
	this->cSprite.Update();
	this->cSprite.setPosition(this->cPos);
}

void CentipedeBody::nextGridFrame() {
	CentipedeSegment::nextGridFrame();
}

void CentipedeBody::Collision(Bullet* other) {
	this->pCentipede->requestSplit(this);
	ScoreManager::SendScoreCommand(this->pDeath);
	SoundManager::PlayKillSound();
	AnimationManager::RequestRegenerationAnimation(this->cPos);
	//MarkForDestroy();
}

void CentipedeBody::Destroy() {
	//Trace::out("Destroyed BODY %p\n", this);
	DeregisterCollision<CentipedeBody>(*this);
}

CentipedeBody::~CentipedeBody() {
	delete pDeath;
}