#include "CentipedeHead.h"
#include "MoveState.h"
#include "MoveState_DownSwitchLeft.h"
#include "MoveState_DownSwitchRight.h"
#include "CentipedeBody.h"
#include "Centipede.h"
#include "ScoreManager.h"
#include "ScoreCommand.h"
#include "SoundManager.h"
#include "AnimationManager.h"

CentipedeHead::CentipedeHead() {
	this->bitmap = &ResourceManager::GetTextureBitmap("centipedeHeadSprite");
	this->cSprite = AnimatedSprite(ResourceManager::GetTexture("centipedeHeadSprite"), 8, 2, 8, 0, 7, true, true);
	//this->Initialize();
	this->cSprite.setScale(2.0f, 2.0f);
	this->frameCounter = 0;

	this->pDeath = ScoreManager::GetScoreCommand(ScoreManager::SCORE_EVENTS::CentipedeHeadDeath);

	this->SetCollider(this->cSprite, *bitmap);
}

void CentipedeHead::Initialize(Centipede *pCent, int spd, int gpos) {
	CentipedeSegment::Initialize(pCent, spd, gpos);
	RegisterCollision<CentipedeHead>(*this);
	this->gridPos.x = gpos;
	this->gridPos.y = -1;
	this->cPos = sf::Vector2f(gridPos.x * cn::GRID_SEGMENT_WIDTH, gridPos.y * cn::GRID_SEGMENT_HEIGHT);
	this->frameCounter = 0;
	//this is not its job, but will change soon
	if (rand() % 2 == 0)
		this->pMoveState = &MoveFSM::SWITCH_LEFT_DOWN;
	else
		this->pMoveState = &MoveFSM::SWITCH_RIGHT_DOWN;
	
}

void CentipedeHead::DeInitialize() {
	this->gridPos.x = 3;
	this->gridPos.y = 3;
	this->pMoveState = nullptr;
	this->cPos.x = 33;
	this->cPos.y = 33;
	this->pCentipede = nullptr;
	this->speed = 10;
	this->frameCounter = 1;
}

void CentipedeHead::nextGridFrame() {
	CentipedeSegment::nextGridFrame();
	this->pMoveState = this->pMoveState->GetNextState(this);
}

void CentipedeHead::assureGridFrame() {
	this->pMoveState = this->pMoveState->GetNextState(this);
}


void CentipedeHead::copyDataFromBody(CentipedeBody* cBodData) {
	//Trace::out("Upgrading Body: %p to a head: %p\n", cBodData, this);
	pCentipede = cBodData->getMother();
	this->gridPos = cBodData->getGridPos();
	this->cPos = cBodData->getPos();
	this->pMoveState = cBodData->getMoveState();
	this->speed = cBodData->getSpeed();
}

void CentipedeHead::Collision(Bullet* other) {
	this->pCentipede->requestHeadKill();
	ScoreManager::SendScoreCommand(this->pDeath);
	SoundManager::PlayKillSound();
	AnimationManager::RequestRegenerationAnimation(this->cPos);
	//MarkForDestroy();
}

void CentipedeHead::Destroy() {
	//Trace::out("Destroyed HEAD %p\n", this);
	//this->DeInitialize();
	DeregisterCollision<CentipedeHead>(*this);
}

CentipedeHead::~CentipedeHead() {
	delete pDeath;
}