#include "CentipedeManager.h"
#include "Definitions.h"
#include "Centipede.h"
#include "CentipedeObjectPool.h"
#include "CentipedeSegmentObjectPool.h"
#include "CentipedeHead.h"
#include "CentipedeBody.h"
#include "WaveManager.h"
#include "Wave.h"
#include "SoundManager.h"

CentipedeManager* CentipedeManager::instance = nullptr;

CentipedeManager::CentipedeManager() {
	this->framesUntilGridCheck = WaveManager::GetCurrentWave()->getFramesPerUpdate()-1;
	this->pPool = new CentipedeObjectPool;
	this->pSegPool = new CentipedeSegmentObjectPool;
	this->centipedeHasBeenSpawnedThisWave = false;
	this->requestedNewCentipede = false;
	this->headsToSpawn = 0;
	this->soloFramesUntilCheck = WaveManager::GetCurrentWave()->getFramesPerUpdateHead() - 1;
}

void CentipedeManager::privUpdate() {
	framesUntilGridCheck++;
	soloFramesUntilCheck++;
	if (framesUntilGridCheck == WaveManager::GetCurrentWave()->getFramesPerUpdate()) {
		framesUntilGridCheck = 0;
		//for (std::vector<Centipede*>::iterator it = centipedeList.begin(); it != formerEnd; ++it) {
		//	//Trace::out("Number of Centipedes: %d\n",centipedeList.size());
		//	(*it)->Update();
		//}
  		int formerSize = centipedeList.size();
		for (int i = 0; i < formerSize; i++) {
			centipedeList.at(i)->SplitUpdate();
		}
		int formerHeadSize = headList.size();
		for (int i = 0; i < formerHeadSize; i++) {
			headList.at(i)->SplitUpdate();
		}

		for (std::vector<Centipede*>::iterator it = toBeRecycledList.begin(); it != toBeRecycledList.end(); ++it) {
			this->privRecycleCentipede(*it);
		}
		toBeRecycledList.clear();

		for (std::vector<Centipede*>::iterator it = centipedeList.begin(); it != centipedeList.end(); ++it) {
			(*it)->Update();
		}
		if (this->requestedNewCentipede) {
			privSpawnCentipede(requestedNewCentipedeSpeed, requestedNewCentipedeLength, spawnRandom);
			requestedNewCentipede = false;
			centipedeHasBeenSpawnedThisWave = true;
		}
		
		
	}
	if (soloFramesUntilCheck == WaveManager::GetCurrentWave()->getFramesPerUpdateHead()) {
		soloFramesUntilCheck = 0;
		int formerSize = headList.size();
		for (int i = 0; i < formerSize; i++) {
			headList.at(i)->Update();
		}
		if (headsToSpawn > 0) {
			while (headsToSpawn > 0) {
				privSpawnCentipedeHead(WaveManager::GetCurrentWave()->getSoloHeadSpeed(), true);
				headsToSpawn--;
			}
		}
	}
}

void CentipedeManager::privSpawnCentipede(int spd, int length, bool rnd) {
	Centipede* temp = pPool->requestCentipede();
	temp->Initialize(spd, length, rnd);
	centipedeList.push_back(temp);
}

void CentipedeManager::privSpawnCentipedeHead(int spd, bool rnd) {
	Centipede* tmp = pPool->requestCentipede();
	tmp->Initialize(spd, 1, rnd);
	headList.push_back(tmp);
}


void CentipedeManager::privRecycleCentipede(Centipede* obj) {
	std::vector<Centipede*>::iterator it;
	bool foundInCentipedeList = false;
	for (it = centipedeList.begin(); it != centipedeList.end(); ++it) {
		if (*it == obj) {
			foundInCentipedeList = true;
			goto foundIt;
		}
	}
	for (it = headList.begin(); it != headList.end(); ++it) {
		if (*it == obj) {
			break;
		}
	}
foundIt:	
	pPool->recycleCentipede(*it);
	if (foundInCentipedeList)
		this->centipedeList.erase(it);
	else
		this->headList.erase(it);
}


void CentipedeManager::privToBeRecycled(Centipede* obj) {
	this->toBeRecycledList.push_back(obj);
}

CentipedeHead* CentipedeManager::privUpgradeToHead(CentipedeBody* cBod) {
	CentipedeHead* ret = pSegPool->requestCentipedeHead();
	ret->SetExternalManagement(RecycleHead);
	ret->copyDataFromBody(cBod);
	ret->RegisterCollision<CentipedeHead>(*ret);
	return ret;
}

CentipedeHead* CentipedeManager::privRequestHead(Centipede* pMother, int spd, int gpos) {
	CentipedeHead* ret = pSegPool->requestCentipedeHead();
	ret->SetExternalManagement(RecycleHead);
	ret->Initialize(pMother, spd, gpos);
	return ret;
}

CentipedeBody* CentipedeManager::privRequestBody(Centipede* pMother, int spd, int gpos) {
	CentipedeBody* ret = pSegPool->requestCentipedeBody();
	ret->SetExternalManagement(RecycleBody);
	ret->Initialize(pMother, spd, gpos);
	return ret;
}

void CentipedeManager::privRecycleHead(GameObject* obj) {
	//Trace::out("Sent Head to factory: %p\n", obj);
	static_cast<CentipedeHead*>(obj)->DeInitialize();
	this->pSegPool->recycleCentipedeHead(static_cast<CentipedeHead*>(obj));
}

void CentipedeManager::privRecycleBody(GameObject* obj) {
	//Trace::out("Sent Body to factory %p\n", obj);
	static_cast<CentipedeBody*>(obj)->DeInitialize();
	this->pSegPool->recycleCentipedeBody(static_cast<CentipedeBody*>(obj));
}

Centipede* CentipedeManager::privGetNewCentipede_toInsert() {
	Centipede* temp = pPool->requestCentipede();
	centipedeList.push_back(temp);
	return temp;
}


void CentipedeManager::recycleAll() {
	
}

void CentipedeManager::privTerminate() {
	delete instance;
}

CentipedeManager::~CentipedeManager() {
	while (!this->centipedeList.empty()) {
		delete this->centipedeList.back();
		this->centipedeList.pop_back();
	}
	while (!this->headList.empty()) {
		delete this->headList.back();
		this->headList.pop_back();
	}
	delete pPool;
	delete pSegPool;
}

void CentipedeManager::killAllCentipedes() {
	const std::vector<Centipede*> myCentipedes = centipedeList;
	std::vector<Centipede*>::const_iterator cit = myCentipedes.begin();
	while (cit != myCentipedes.end()) {
		(*cit)->KillAll();
		++cit;
	}
	const std::vector<Centipede*> myCentipedes2 = headList;
	cit = myCentipedes2.begin();
	while (cit != myCentipedes2.end()) {
		(*cit)->KillAll();
		++cit;
	}
}

void CentipedeManager::privEndRound() {
	killAllCentipedes();
	SoundManager::StopCentipedeSound();
}

bool CentipedeManager::privDoesCentipedeExist() {
	//for now
	return !(this->centipedeHasBeenSpawnedThisWave && this->centipedeList.empty() && this->headList.empty());
}

void CentipedeManager::privRequestCentipedeSpawn(int spd, int len, bool rnd) {
	requestedNewCentipede = true;
	requestedNewCentipedeSpeed = spd;
	requestedNewCentipedeLength = len;
	spawnRandom = rnd;
	SoundManager::PlayCentipedeSound();
}

void CentipedeManager::privSpawnSoloHead(int count) {
	headsToSpawn += count;
}