#ifndef CENTIPEDE_MANAGER_H
#define CENTIPEDE_MANAGER_H

#include "Game Components/TEAL/CommonElements.h"
#include <vector>
#include "SoundManager.h"

class Centipede;
class CentipedeObjectPool;
class CentipedeSegmentObjectPool;
class CentipedeHead;
class CentipedeBody;


class CentipedeManager
{
private:
	CentipedeManager();
	CentipedeManager(const CentipedeManager&) = default;
	CentipedeManager& operator=(const CentipedeManager&) = default;
	~CentipedeManager();
	static CentipedeManager* instance;
	static CentipedeManager& getInstance() { 
		if (!instance)
			instance = new CentipedeManager();
		return *instance;
	}
	void privUpdate();
	void privSpawnCentipede(int, int, bool);
	void privSpawnCentipedeHead(int, bool);
	void privRecycleCentipede(Centipede*);
	void privTerminate();
	void recycleAll();
	CentipedeHead* privRequestHead(Centipede*, int, int);
	CentipedeBody* privRequestBody(Centipede*, int, int);
	void privRecycleHead(GameObject*);
	void privRecycleBody(GameObject*);
	Centipede* privGetNewCentipede_toInsert();
	CentipedeHead* privUpgradeToHead(CentipedeBody*);
	void privToBeRecycled(Centipede*);
	void privEndRound();
	void killAllCentipedes();
	bool privDoesCentipedeExist();
	void privRequestCentipedeSpawn(int, int, bool);
	void privSpawnSoloHead(int);
public:
	static void Update() { getInstance().privUpdate(); }
	static void SpawnCentipede(int spd, int len, bool rnd = false) { getInstance().privRequestCentipedeSpawn(spd, len, rnd);	}
	static void SpawnSoloCentipede(int cnt) { getInstance().privSpawnSoloHead(cnt); }
	static void RecycleCentipede(Centipede* go) { getInstance().privRecycleCentipede(go); }
	static void Terminate() { getInstance().privTerminate(); }
	static CentipedeHead* RequestHead(Centipede* obj, int spd, int gpos) { return getInstance().privRequestHead(obj, spd, gpos); }
	static CentipedeBody* RequestBody(Centipede* obj, int spd, int gpos) { return getInstance().privRequestBody(obj, spd, gpos); }
	static void RecycleHead(GameObject* obj) { getInstance().privRecycleHead(obj); }
	static void RecycleBody(GameObject* obj) { getInstance().privRecycleBody(obj); }
	static Centipede* GetNewCentipede_toInsert() { return getInstance().privGetNewCentipede_toInsert(); }
	static CentipedeHead* UpgradeToHead(CentipedeBody* cb) { return getInstance().privUpgradeToHead(cb); }
	static void ToBeRecycled(Centipede* obj) { getInstance().privToBeRecycled(obj); }
	static void MarkForImmediateRecycle(Centipede* obj) { getInstance().privRecycleCentipede(obj); }
	static void EndRound() { getInstance().privEndRound(); }
	static bool DoesCentipedeExist() { return getInstance().privDoesCentipedeExist(); }
	static void NewWave() { getInstance().centipedeHasBeenSpawnedThisWave = false; }
private:
	unsigned char framesUntilGridCheck;
	unsigned char soloFramesUntilCheck;
	std::vector<Centipede*> centipedeList;
	std::vector<Centipede*> headList;
	std::vector<Centipede*> toBeRecycledList;
	CentipedeObjectPool* pPool;
	CentipedeSegmentObjectPool* pSegPool;
	bool requestedNewCentipede;
	int requestedNewCentipedeSpeed;
	int requestedNewCentipedeLength;
	int headsToSpawn;
	bool spawnRandom;
	bool centipedeHasBeenSpawnedThisWave;
};

#endif