#include "CritterController.h"
#include "WaveManager.h"
#include "Wave.h"
#include "MushroomFieldManager.h"
#include "SoundManager.h"

CritterController::CritterController() {
	frameTimer = 0;
}

CritterController* CritterController::instance = nullptr;



void CritterController::privSpawnScorpion() {
	if (WaveManager::GetCurrentWave()->canSpawnScorpion()) {
		liveScorpions.push_back(CritterFactory::requestScorpion());
		SoundManager::PlayScorpionSound();
	}
		
}

void CritterController::privSpawnFlea() {
	if (WaveManager::GetCurrentWave()->canSpawnFlea()) {
		liveFleas.push_back(CritterFactory::requestFlea());
		SoundManager::PlayFleaSound();
	}
		
}

void CritterController::privSpawnSpider() {
	if (WaveManager::GetCurrentWave()->canSpawnSpider()) {
		liveSpiders.push_back(CritterFactory::requestSpider());
		SoundManager::PlaySpiderSound();
	}
		
}

void CritterController::privCatchScorpionDeath(Scorpion* deadScorp) {
	std::vector<Scorpion*>::iterator it;
	for (it = liveScorpions.begin(); it != liveScorpions.end(); ++it) {
		if (*it == deadScorp)
			break;
	}
	liveScorpions.erase(it);
	//CritterFactory::recycleScorpion(deadScorp);
	if (liveScorpions.empty())
		SoundManager::StopScorpionSound();
}

void CritterController::privCatchFleaDeath(Flea* deadFlea) {
	std::vector<Flea*>::iterator it;
	for (it = liveFleas.begin(); it != liveFleas.end(); ++it) {
		if (*it == deadFlea)
			break;
	}
	liveFleas.erase(it);
	//CritterFactory::recycleFlea(deadFlea);
}

void CritterController::privCatchSpiderDeath(Spider* deadSpider) {
	std::vector<Spider*>::iterator it;
	for (it = liveSpiders.begin(); it != liveSpiders.end(); ++it) {
		if (*it == deadSpider)
			break;
	}
	liveSpiders.erase(it);
	if (liveSpiders.empty())
		SoundManager::StopSpiderSound();
	//CritterFactory::recycleSpider(deadSpider);
}

void CritterController::killAllCritters() {
	const std::vector<Spider*> mySpiders = liveSpiders;
	std::vector<Spider*>::const_iterator spit = mySpiders.begin();
	while (spit != mySpiders.end()) {
		(*spit)->MarkForDestroy();
		++spit;
	}
	const std::vector<Flea*> myFleas = liveFleas;
	std::vector<Flea*>::const_iterator flit = myFleas.begin();
	while (flit != myFleas.end()) {
		(*flit)->MarkForDestroy();
		++flit;
	}
	const std::vector<Scorpion*> myScorpions = liveScorpions;
	std::vector<Scorpion*>::const_iterator scit = myScorpions.begin();
	while (scit != myScorpions.end()) {
		(*scit)->MarkForDestroy();
		++scit;
	}
}

void CritterController::privNewRound() {
	
}

void CritterController::privEndRound() {
	killAllCritters();
}

void CritterController::privUpdate() {
	frameTimer++;
	if (frameTimer == cn::CRITTER_TIMER) {
		frameTimer = 0;
		//Trace::out("TRY HERE?\n");
		if (rand() < WaveManager::GetCurrentWave()->getSpiderSpawnChance() * RAND_MAX) {
			if (liveSpiders.empty()) {
				this->privSpawnSpider();
				
			}
		}
		if (rand() < (int)(WaveManager::GetCurrentWave()->getScorpionSpawnChance() * (float)RAND_MAX)) {
			if (liveScorpions.empty()) {
				this->privSpawnScorpion();
				
			};
		}
	}
	if (MushroomFieldManager::DoWeNeedAFlea()) {
		if (liveFleas.empty()) {
			this->privSpawnFlea();
		}
	}
}

void CritterController::privTerminate() {
	delete instance;
}