#ifndef CRITTER_CONTROLLER_H
#define CRITTER_CONTROLLER_H

#include "Game Components/TEAL/CommonElements.h"
#include "Critter.h"
#include "MushroomField.h"
#include "CritterFactory.h"
#include "Scorpion.h"
#include "Flea.h"
#include "Spider.h"
#include <vector>

class MushroomField;
class Scorpion;
class Flea;
class Spider;
class CritterFactory;

class CritterController
{
private:
	CritterController();
	CritterController(const CritterController&) = delete;
	CritterController& operator=(const CritterController&) = delete;
	~CritterController() = default;
	static CritterController* instance;
	static CritterController& getInstance() {
		if (!instance)
			instance = new CritterController();
		return *instance;
	}

	void privSpawnScorpion();
	void privSpawnFlea();
	void privSpawnSpider();
	void privCatchScorpionDeath(Scorpion*);
	void privCatchFleaDeath(Flea*);
	void privCatchSpiderDeath(Spider*);

	void privNewRound();
	void privEndRound();
	void privUpdate();
	void privTerminate();
public:
	static void spawnScorpion() { getInstance().privSpawnScorpion(); }
	static void spawnFlea() { getInstance().privSpawnFlea(); }
	static void spawnSpider() { getInstance().privSpawnSpider(); }
	static void catchScorpionDeath(Scorpion* so) { getInstance().privCatchScorpionDeath(so); }
	static void catchFleaDeath(Flea* fo) { getInstance().privCatchFleaDeath(fo); }
	static void catchSpiderDeath(Spider* so) { getInstance().privCatchSpiderDeath(so); }

	static void newRound() { getInstance().privNewRound(); }
	static void EndRound() { getInstance().privEndRound(); }
	static void Update() { getInstance().privUpdate(); }
	static void Terminate() { getInstance().privTerminate(); }
	static bool ExistsScorpion() { return !(getInstance().liveScorpions.empty()); }
	static bool ExistsSpider() { return !(getInstance().liveSpiders.empty()); }
private:
	int frameTimer;
	std::vector<Scorpion*> liveScorpions;
	std::vector<Spider*> liveSpiders;
	std::vector<Flea*> liveFleas;
	void killAllCritters();
};

#endif

