#ifndef CRITTER_OBJECT_POOL_H
#define CRITTER_OBJECT_POOL_H

#include "Game Components/TEAL/CommonElements.h"
#include "Scorpion.h"
#include "Flea.h"
#include "Spider.h"
#include <stack>

class Scorpion;
class Flea;
class Spider;
class ScorpionFactory;
class MushroomField;
class CritterController;


class CritterObjectPool
{
private:
	CritterObjectPool();
	CritterObjectPool(const CritterObjectPool&) = delete;
	CritterObjectPool& operator=(const CritterObjectPool&) = delete;
	~CritterObjectPool();
	static CritterObjectPool* instance;
	static CritterObjectPool& getInstance() {
		if (!instance)
			instance = new CritterObjectPool();
		return *instance;
	}
	Scorpion* privGetScorpion();
	void privReturnScorpion(Scorpion*);
	//fleas
	Flea* privGetFlea();
	void privReturnFlea(Flea*);
	//spiders
	Spider* privGetSpider();
	void privReturnSpider(Spider*);

	void privTerminate();
public:
	//scorpions
	static Scorpion* getScorpion() { return getInstance().privGetScorpion(); }
	static void returnScorpion(Scorpion* so) { getInstance().privReturnScorpion(so); }
	//fleas
	static Flea* getFlea() { return getInstance().privGetFlea(); }
	static void returnFlea(Flea* fo) { getInstance().privReturnFlea(fo); }
	//spiders
	static Spider* getSpider() { return getInstance().privGetSpider(); }
	static void returnSpider(Spider* so) { getInstance().privReturnSpider(so); }

	static void Terminate() { getInstance().privTerminate(); }

private:
	std::stack<Scorpion*> recycledScoprions;
	std::stack<Flea*> recycledFleas;
	std::stack<Spider*> recycledSpiders;
};

#endif