#include "Flea.h"
#include "MushroomFieldManager.h"
#include "ScoreCommand.h"
#include "ScoreManager.h"
#include "SoundManager.h"
#include "AnimationManager.h"

Flea::Flea() {
	this->bitmap = &ResourceManager::GetTextureBitmap("fleaSprite");
	this->cSprite = AnimatedSprite(ResourceManager::GetTexture("fleaSprite"), 2, 2, 8, 0, 1, true, true);

	pDeath = ScoreManager::GetScoreCommand(ScoreManager::SCORE_EVENTS::FleaKilled);

	this->SetCollider(this->cSprite, *bitmap);
}

void Flea::Initialize() {
	this->hp = 2;
	this->gridX = rand() % cn::MUSHROOM_FIELD_COLUMNS;
	this->gridY = 0;
	this->cPos.x = (float)(gridX * (char)cn::PLAYER_SIZE);
	this->cPos.y = 0.0f;
	this->updateAI = &fullHealthAI;
	this->untilNextGridSpaceY = 0;
	RegisterCollision<Flea>(*this);
}

void Flea::Update() {

	
	if (cPos.y > cn::SCREEN_HEIGHT)
		MarkForDestroy();

	this->updateAI(this);
	Critter::Update();
}

void Flea::Destroy() {
	CritterController::catchFleaDeath(this);
	
	DeregisterCollision<Flea>(*this);
}

void Flea::fullHealthAI(Flea* inst) {
	inst->cPos.y += cn::FLEA_SPEED;
	inst->untilNextGridSpaceY += (char)cn::FLEA_SPEED;
	if (inst->untilNextGridSpaceY > cn::PLAYER_SIZE) {
		inst->untilNextGridSpaceY -= cn::PLAYER_SIZE;
		inst->spawnMushroom();
		inst->gridY++;
		//Trace::out("Grid Position: (%d,%d)\n", inst->gridX, inst->gridY);
	}

}

void Flea::lowHealthAI(Flea *inst) {
	inst->cPos.y += cn::FLEA_ENHANCED_SPEED;
}

void Flea::Collision(Bullet *other) {
	hp--;
	if (hp == 1) {
		updateAI = &lowHealthAI;
	}
	else {
		ScoreManager::SendScoreCommand(this->pDeath);
		SoundManager::PlayKillSound();
		AnimationManager::RequestRegenerationAnimation(this->cPos);
		MarkForDestroy();
	}
}

void Flea::spawnMushroom() {
	if (rand() < cn::FLEA_MUSHROOM_SPAWN_CHANCE * RAND_MAX) {
		MushroomFieldManager::AddMushroomToCurrentField(sf::Vector2<char>(this->gridX, this->gridY));
	}
}

Flea::~Flea() {
	delete pDeath;
}