#include "GameManager.h"
#include "OffsetCollection.h"
#include "CentipedeManager.h"
#include "MushroomFieldManager.h"
#include "MushroomController.h"
#include "MushroomObjectPool.h"
#include "WaveManager.h"
#include "Wave.h"
#include "ProfileManager.h"
#include "Profile.h"
#include "ScoreManager.h"
#include "GlyphSheet.h"
#include "ScoreDisplay.h"
#include "ScoreFont.h"
#include "SoundManager.h"
#include "AnimationManager.h"
#include "BulletManager.h"
#include "InputController.h"
#include "GameMode.h"
#include "GameMode1P.h"
#include "GameMode2P.h"
#include "GameModeAttractor.h"
#include "InvisibleBarrier.h"
#include "HighScoreManager.h"
#include "ShipAI.h"


GameManager* GameManager::instance = nullptr;

GameManager::GameManager() {
	privUpdate = &MainLoop;
	privDrawEvents = &MainDraw;
	frameHolder = 0;
	colHolder = 0;
	EndOfRound = false;
	DoubleHS = false;
	
	MushroomFieldManager::CreateNewField();
	this->onePlayer = new GameMode1P();
	this->twoPlayer = new GameMode2P();
	this->attractorPlayer = new GameModeAttractor();
	currentGameMode = nullptr;
	

	
	//for now defaulting to keyboard, can change this later.
	pInput = new InputController(CONTROLLER_TYPE::AI);
	//factory and pool scorpion
	//set critters stuff
	SoundManager::ChangeVolume(cn::VOLUME);

	this->privNewGameAI();
}

void GameManager::privNewGame1P() {
	MushroomFieldManager::NewGame();
	this->currentGameMode = this->onePlayer;
	this->EndCurrentGame();
	ProfileManager::Initialize1P();
	this->pInput->changeInputType(CONTROLLER_TYPE::KEYBOARD);
	SoundManager::UnMute();
	this->privNewGame();
}

void GameManager::privNewGame2P() {
	MushroomFieldManager::NewGame();
	this->currentGameMode = this->twoPlayer;
	this->EndCurrentGame();
	ProfileManager::Initialize2P();
	this->pInput->changeInputType(CONTROLLER_TYPE::KEYBOARD);
	SoundManager::UnMute();
	this->privNewGame();
}

void GameManager::privNewGameAI() {
	this->currentGameMode = this->attractorPlayer;
	MushroomFieldManager::NewGameAI();
	ProfileManager::InitializeAI();
	this->pInput->changeInputType(CONTROLLER_TYPE::AI);
	SoundManager::Mute();
	this->privNewGame();
}

GameManager::~GameManager() {
	/*this->pScorpionObjectPool->Terminate();
	this->pScorpionFactory->Terminate();*/
	delete onePlayer;
	delete twoPlayer;
	delete attractorPlayer;
}


void GameManager::EndCurrentGame() {
	CritterController::EndRound();
	CentipedeManager::EndRound();
	//temp, should not be here...
	MushroomFieldManager::NewGame();
}

void GameManager::privTerminate() {
	CritterObjectPool::Terminate();
	CritterFactory::Terminate();
	CritterController::Terminate();
	MushroomController::Terminate();
	MushroomObjectPool::Terminate();
	CentipedeManager::Terminate();
	OffsetCollection::Terminate();
	MushroomFieldManager::Terminate();
	WaveManager::Terminate();
	ScoreManager::Terminate();
	ProfileManager::Terminate();
	SoundManager::Terminate();
	AnimationManager::Terminate();
	BulletManager::Terminate();
	ScoreDisplay::Terminate();
	HighScoreManager::Terminate();
	ShipAI::Terminate();
	//delete textSheet;
	delete instance;

}


void GameManager::privMainLoop() {
	CentipedeManager::Update();
	CritterController::Update();
	if (CentipedeManager::DoesCentipedeExist() == false) {
		WaveManager::NextWave();
		ProfileManager::GetCurrentPlayer()->AdvanceWave();
		CentipedeManager::NewWave();
		CentipedeManager::SpawnCentipede(WaveManager::GetCurrentWave()->getCentipedeSpeed(), WaveManager::GetCurrentWave()->getCentipedeLength());
		CentipedeManager::SpawnSoloCentipede(WaveManager::GetCurrentWave()->getSoloHeadCount());
	}
	//Trace::out("Does Centipede Exist: %s\n", CentipedeManager::DoesCentipedeExist() ? "TRUE" : "FALSE");
	ScoreManager::ProcessScores();
	AnimationManager::Update();
}

void GameManager::privRegenLoop() {
	LoadDeathScreen();
	ScoreManager::ProcessScores();
	AnimationManager::Update();
}

void GameManager::privHSLoop() {
	//nothing to update here. nothing should update.
}


void GameManager::privExitHSLoop() {
	if (this->DoubleHS) {
		this->DoubleHS = false;
		ProfileManager::ResetPlayer1Score();
	}
	else {
		privUpdate = &MainLoop;
		privDrawEvents = &MainDraw;
		this->privNewGameAI();
	}
}

void GameManager::privMainDraw() {
	this->currentGameMode->Update();
	ScoreDisplay::DrawMiscText();
}

void GameManager::privHSDraw() {
	ScoreDisplay::DrawNewHSMode();
	this->privMainDraw();
}


void GameManager::privNewGame() {
	ProfileManager::NewGame();
	WaveManager::SetWave(0);
	
	this->StartNewRound();
}

void GameManager::privNewRound() {
	ProfileManager::GetCurrentPlayer()->LoseOneLife();
	SoundManager::PlayDeathSound();
	AnimationManager::RequestDeathAnimation(ProfileManager::GetCurrentPlayer()->GetShipPositon());
	CritterController::EndRound();
	CentipedeManager::EndRound();
	if (!ProfileManager::ExistsLivePlayer()) {
		if (HighScoreManager::CheckForNewHighScores()) {
			//if (currentGameMode != nullptr) delete this->currentGameMode;
			//this->currentGameMode = new GameModeNewHighScore();
			
			privUpdate = &HSLoop;
			privDrawEvents = &HSDraw;
			ScoreDisplay::ResetHighScoreText();
			this->DoubleHS = HighScoreManager::CheckForNewHighScoresBoth();
			MushroomFieldManager::GetCurrentField()->HighScoreDelete();
			this->pInput->changeInputType(CONTROLLER_TYPE::HIGHSCORE);
			ProfileManager::NewGame();
			//ScoreDisplay::UpdateHighScores();
		} else 
			this->privNewGameAI();
	}
	else {
		ProfileManager::EndRound();
		privUpdate = &RegenLoop;
	}
}

void GameManager::LoadDeathScreen() {
	frameHolder++;
	if (frameHolder == 4) {
		frameHolder = 0;
		MushroomFieldManager::RegenerateCurrentColumn(colHolder);
		
		colHolder++;
	}
	if (colHolder >= cn::MUSHROOM_FIELD_COLUMNS) {
		colHolder = 0;
		currentGameMode->SwapPlayers();
		privUpdate = &MainLoop;
		this->StartNewRound();
	}
}

void GameManager::StartNewRound() {
	EndOfRound = false;
	BulletManager::KillAllBullets();
	ProfileManager::NewRound();
	CentipedeManager::NewWave();
	CentipedeManager::SpawnCentipede(WaveManager::GetCurrentWave()->getCentipedeSpeed(), WaveManager::GetCurrentWave()->getCentipedeLength());
	CentipedeManager::SpawnSoloCentipede(WaveManager::GetCurrentWave()->getSoloHeadCount());
	MushroomFieldManager::NewRound();
}

void GameManager::privNewRoundProcessing() {
}


//will finish at a later date when necessary. This allows the player to change gamemodes;
void GameManager::privChangeGameMode(GAMEMODE gm) {
	//this->pInput->changeInputType(CONTROLLER_TYPE::SOMETHING);
	this->privNewGame();
}