#ifndef GAME_MANAGER_H
#define GAME_MANAGER_H

#include "Game Components/TEAL/CommonElements.h"
#include "InputController.h"
#include "MushroomField.h"
#include "Definitions.h"
#include "Critter.h"
#include "CritterController.h"




class InputController;
class MushroomField;
class CritterController;
class BulletFactory;
class CritterFactory;
class CritterObjectPool;
class MushroomObjectPool;
class MushroomController;
class GlyphSheet;
class GameMode;


enum class GAMEMODE: unsigned char {
	SINGLE_PLAYER,
	MULTIPLAYER,
	ATTRACTOR
};


class GameManager
{
private:
	GameManager();
	GameManager(const GameManager&) = delete;
	GameManager& operator=(const GameManager&) = delete;
	~GameManager();
	static GameManager* instance;
	static GameManager& getInstance() {
		if (!instance)
			instance = new GameManager();
		return *instance;
	}
private:
	//private non-static functions
	void (*privUpdate)();
	void (*privDrawEvents)();
	void privNewGame();
	void privChangeGameMode(GAMEMODE);
	void privTerminate();
	void privNewRound();
	void privNewRoundProcessing();
	void LoadDeathScreen();
	void StartNewRound();
	void privNewGame1P();
	void privNewGame2P();
	void privNewGameAI();
	void EndCurrentGame();
	void privMainLoop();
	void privRegenLoop();
	void privHSLoop();
	void privMainDraw();
	void privHSDraw();
	void privExitHSLoop();
public:
	//public static functions
	static void Update() { getInstance().privUpdate(); }
	static void NewGame() { getInstance().privNewGame(); }
	static void changeGameMode(GAMEMODE gm) { getInstance().privChangeGameMode(gm); }
	static void Terminate() { getInstance().privTerminate(); }
	static void NewRound() { getInstance().privNewRound(); }
	static void NewGame1P() { getInstance().privNewGame1P(); }
	static void NewGame2P() { getInstance().privNewGame2P(); }
	static void DrawEvents() { getInstance().privDrawEvents(); }
	static void EndHSLoop() { getInstance().privExitHSLoop(); }
private:
	
	InputController* pInput;
	int frameHolder;
	int colHolder;
	bool EndOfRound;
	bool DoubleHS;
	int headsToDelay;

	GameMode* currentGameMode;
	GameMode* onePlayer;
	GameMode* twoPlayer;
	GameMode* attractorPlayer;

	static void MainLoop() { getInstance().privMainLoop(); }
	static void RegenLoop() { getInstance().privRegenLoop(); }
	static void HSLoop() { getInstance().privHSLoop(); }

	static void MainDraw() { getInstance().privMainDraw(); }
	static void HSDraw() { getInstance().privHSDraw(); }
};

#endif
