#include "HighScoreManager.h"
#include "ProfileManager.h"
#include "Profile.h"

#include <iostream>
#include <string>
#include <fstream>

HighScoreManager* HighScoreManager::instance = nullptr;

HighScoreManager::HighScoreManager() {
	ReadInHighScores("high_scores.txt");
	highestScore = highScoresList.front().first;
}

HighScoreManager::~HighScoreManager() {

}

void HighScoreManager::privTerminate() {
	//put new high scores to file.
	this->PopulateFilesOnExit();
	delete instance;
}

void HighScoreManager::PopulateFilesOnExit() {
	std::ofstream myFile;
	myFile.open("high_scores.txt");
	for (std::list<std::pair<int, char[4]>>::iterator it = highScoresList.begin(); it != highScoresList.end(); ++it) {
		myFile << (*it).first << " " << (*it).second << "\n";
	}
	myFile.close();
}

void HighScoreManager::privLoadScores() {
	ReadInHighScores("high_scores.txt");
	highestScore = highScoresList.front().first;
}

void HighScoreManager::ReadInHighScores(char* fileName) {
	std::string line;
	std::ifstream myFile(fileName);

	std::list<std::pair<int, char[4]>>::iterator it;
	if (myFile.is_open()) {
		while (getline(myFile, line)) {
			int score;
			char inits[3];
			size_t delim_pos = line.find(' ');
			score = atoi(line.substr(0, delim_pos).c_str());
			inits[0] = line.at(delim_pos + 1);
			inits[1] = line.at(delim_pos + 2);
			inits[2] = line.at(delim_pos + 3);
			std::pair<int, char[4]> a;
			a.first = score;
			a.second[0] = inits[0];
			a.second[1] = inits[1];
			a.second[2] = inits[2];
			highScoresList.push_back(a);
		}
		myFile.close();
		highScoresList.resize(8);
	}
	
}

bool HighScoreManager::privCheckForNewHighScores() {
	int scoreP1 = ProfileManager::GetPlayer1Score();
	int scoreP2 = ProfileManager::GetPlayer2Score();

	for (std::list<std::pair<int, char[4]>>::iterator it = highScoresList.begin(); it != highScoresList.end(); ++it) {
		if (scoreP1 > (*it).first) {
			return true;
		}
	}
	for (std::list<std::pair<int, char[4]>>::iterator it = highScoresList.begin(); it != highScoresList.end(); ++it) {
		if (scoreP2 > (*it).first) {
			return true;
		}
	}
	return false;
}

bool HighScoreManager::privCheckForNewHighScoresBoth() {
	int scoreP1 = ProfileManager::GetPlayer1Score();
	int scoreP2 = ProfileManager::GetPlayer2Score();
	bool p1 = false;
	bool p2 = false;
	for (std::list<std::pair<int, char[4]>>::iterator it = highScoresList.begin(); it != highScoresList.end(); ++it) {
		if (scoreP1 > (*it).first) {
			p1 = true;
		}
	}
	for (std::list<std::pair<int, char[4]>>::iterator it = highScoresList.begin(); it != highScoresList.end(); ++it) {
		if (scoreP2 > (*it).first) {
			p2 = true;
		}
	}
	return (p1 && p2);
}

void HighScoreManager::privAddHighScore(char inits[4]) {
	int scoreP1 = ProfileManager::GetPlayer1Score();
	int scoreP2 = ProfileManager::GetPlayer2Score();
	for (std::list<std::pair<int, char[4]>>::iterator it = highScoresList.begin(); it != highScoresList.end(); ++it) {
		if (scoreP1 > (*it).first) {
			std::pair<int, char[4]> a;
			a.first = scoreP1;
			a.second[0] = inits[0];
			a.second[1] = inits[1];
			a.second[2] = inits[2];
			highScoresList.insert(it, a);
			highScoresList.resize(8);
			return;
		}
	}
	for (std::list<std::pair<int, char[4]>>::iterator it = highScoresList.begin(); it != highScoresList.end(); ++it) {
		if (scoreP2 > (*it).first) {
			std::pair<int, char[4]> a;
			a.first = scoreP2;
			a.second[0] = inits[0];
			a.second[1] = inits[1];
			a.second[2] = inits[2];
			highScoresList.insert(it, a);
			highScoresList.resize(8);
			return;
		}
	}
}


bool HighScoreManager::privHigherThanHighScore(int val) {
	if (val > highestScore) {
		highestScore = val;
		return true;
	}
	return false;
}