#include "InputController.h"
#include "CentipedeManager.h"
#include "GameManager.h"
#include "SoundManager.h"
#include "AnimationManager.h"
#include "ProfileManager.h"
#include "Profile.h"
#include "CritterController.h"
#include "Game Components/TEAL/CommonElements.h"
#include "ScoreDisplay.h"
#include "ShipAI.h"
#include "WaveManager.h"
#include "Wave.h"

InputController::InputController() : InputController(CONTROLLER_TYPE::KEYBOARD)
{
}

//this collection of static functions account for each input type.
//by default, it will choose keyboard.
//this is more efficient than an if-statement every loop as it only checks for the input type upon creation, 
//but it does have the flexibility to change input types on the fly.
void InputController::UpdateKeyboard() {

	//while I know the ifs are necessary, I don't like the interaction of isKeyPressed.
	//I want individual presses instead, will soon find the right keywords to lookup.
	if (sf::Keyboard::isKeyPressed(sf::Keyboard::A)) ProfileManager::GetCurrentPlayer()->moveLeft();
	if (sf::Keyboard::isKeyPressed(sf::Keyboard::D)) ProfileManager::GetCurrentPlayer()->moveRight();
	if (sf::Keyboard::isKeyPressed(sf::Keyboard::W)) ProfileManager::GetCurrentPlayer()->moveUp();
	if (sf::Keyboard::isKeyPressed(sf::Keyboard::S)) ProfileManager::GetCurrentPlayer()->moveDown();
	if (sf::Keyboard::isKeyPressed(sf::Keyboard::Space)) ProfileManager::GetCurrentPlayer()->Shoot();
	
	////these are just here for testing purposes.
	//if (sf::Keyboard::isKeyPressed(sf::Keyboard::Num1)) pCrit->spawnScorpion();
	//if (sf::Keyboard::isKeyPressed(sf::Keyboard::Num2)) pCrit->spawnFlea();
	//if (sf::Keyboard::isKeyPressed(sf::Keyboard::Num3)) pCrit->spawnSpider();
	
}

void InputController::UpdateJoystick() {
	//Trace::out("JOYSTICK RULES!\n");
}

void InputController::UpdateXBOX() {
	//Trace::out("XBOX PWNS!\n");
}

void InputController::UpdateHighScore() {
	
	
}

void InputController::UpdateAI() {
	ShipAI::Update();
	if (sf::Keyboard::isKeyPressed(sf::Keyboard::Num1)) {
		GameManager::NewGame1P();
	}
	if (sf::Keyboard::isKeyPressed(sf::Keyboard::Num2)) {
		GameManager::NewGame2P();
	}
	
}

void InputController::KeyPressed(sf::Keyboard::Key k, bool altKey, bool ctrlKey, bool shiftKey, bool systemKey) {
	if (k == sf::Keyboard::Num6) CritterController::spawnScorpion();
	if (k == sf::Keyboard::Num7) CritterController::spawnFlea();
	if (k == sf::Keyboard::Num8) CritterController::spawnSpider();
	if (k == sf::Keyboard::Num9) CentipedeManager::SpawnCentipede(WaveManager::GetCurrentWave()->getCentipedeSpeed(), WaveManager::GetCurrentWave()->getCentipedeLength());
	if (k == sf::Keyboard::Num0) 
		CentipedeManager::SpawnCentipede(WaveManager::GetCurrentWave()->getSoloHeadSpeed(), 1, true);
	if (k == sf::Keyboard::M) SoundManager::MuteToggle();
	
}

void InputController::TextEntered(sf::Uint32 unichar) {
	ScoreDisplay::ProcessInitials(unichar);
}

InputController::InputController(CONTROLLER_TYPE type) {
	//this is a function pointer that points the update function to the appropriate static function
	if (type == CONTROLLER_TYPE::KEYBOARD) {
		RegisterInput(InputFlags::KeyPressed);
		UpdateProxy = &UpdateKeyboard;
	}
	else if (type == CONTROLLER_TYPE::JOYSTICK) {
		RegisterInput(InputFlags::JoystickButtonPressed);
		UpdateProxy = &UpdateJoystick;
	}
	else if (type == CONTROLLER_TYPE::XBOX) {
		UpdateProxy = &UpdateXBOX;
	}
	else if (type == CONTROLLER_TYPE::AI) {
		RegisterInput(InputFlags::KeyPressed);
		UpdateProxy = &UpdateAI;
	}
	else {
		RegisterInput(InputFlags::KeyPressed | InputFlags::TextEntered);
		UpdateProxy = &UpdateHighScore;
	}
}

void InputController::Update() {
	(*UpdateProxy)();
}

void InputController::changeInputType(CONTROLLER_TYPE type) {
	DeregisterInput(InputFlags::KeyPressed | InputFlags::JoystickButtonPressed | InputFlags::TextEntered);
	if (type == CONTROLLER_TYPE::KEYBOARD) {
		RegisterInput(InputFlags::KeyPressed);
		UpdateProxy = &UpdateKeyboard;
	}
	else if (type == CONTROLLER_TYPE::JOYSTICK) {
		RegisterInput(InputFlags::JoystickButtonPressed);
		UpdateProxy = &UpdateJoystick;
	}
	else if (type == CONTROLLER_TYPE::XBOX) {
		UpdateProxy = &UpdateXBOX;
	}
	else if (type == CONTROLLER_TYPE::AI) {
		RegisterInput(InputFlags::KeyPressed);
		UpdateProxy = &UpdateAI;
	}
	else {
		RegisterInput(InputFlags::KeyPressed | InputFlags::TextEntered);
		UpdateProxy = &UpdateHighScore;
	}
}

