#include "Mushroom.h"
#include "MushroomFieldManager.h"
#include "ScoreManager.h"
#include "ScoreCommand.h"
#include "SoundManager.h"
#include "AnimationManager.h"
#include "Definitions.h"


Mushroom::Mushroom() {
	this->bitmap = &ResourceManager::GetTextureBitmap("mushroomSprite");
	this->mushroomSprite = AnimatedSprite(ResourceManager::GetTexture("mushroomSprite"), 4, 2, 30, 0, 7, false, false);

	pDeath = ScoreManager::GetScoreCommand(ScoreManager::SCORE_EVENTS::MushroomKilled);
	pRegenerate = ScoreManager::GetScoreCommand(ScoreManager::SCORE_EVENTS::MushroomRegenerated);

	this->SetDrawOrder(20);
	SetCollider(mushroomSprite, *bitmap);

}

void Mushroom::Initialize(sf::Vector2f spawnPos) {
	this->mushroomSprite.SetFrame(0);
	pos = spawnPos;
	poisoned = false;
	this->hp = cn::MUSHROOM_MAX_HP;
	RegisterCollision<Mushroom>(*this);
	shouldDraw = true;
}

void Mushroom::Collision(Bullet* other) {
	this->hp--;
	this->updateSprite();
}

void Mushroom::Collision(Spider* other) {
	MushroomFieldManager::RemoveMushroomFromCurrentField(this->pos);
	MarkForDestroy();
}

void Mushroom::Collision(Player* other) {
	
}

void Mushroom::Update() {
	this->mushroomSprite.Update();
	this->mushroomSprite.setPosition(pos);
}

void Mushroom::Draw() {
	if(shouldDraw)
		WindowManager::Window().draw(this->mushroomSprite);
}

void Mushroom::Destroy() {
	
	DeregisterCollision<Mushroom>(*this);
}

void Mushroom::updateSprite() {
	
	if (!hp) {
		MushroomFieldManager::RemoveMushroomFromCurrentField(this->pos);
		ScoreManager::SendScoreCommand(this->pDeath);
		MarkForDestroy();
	}
	else
		this->mushroomSprite.NextFrame();
}

Mushroom::~Mushroom() {
	delete pDeath;
	delete pRegenerate;
}

void Mushroom::poisonThis() {
	if (!this->poisoned)
	{
		delete pDeath;
		pDeath = ScoreManager::GetScoreCommand(ScoreManager::SCORE_EVENTS::MushroomPoisonKilled);
		this->poisoned = true;
		this->mushroomSprite.SetFrame(this->mushroomSprite.GetFrame() + 4);
	}
	
}

void Mushroom::HealToFull() {
	if (hp != cn::MUSHROOM_MAX_HP) {
		ScoreManager::SendScoreCommand(this->pRegenerate);
		SoundManager::PlayRegenerateSound();
		AnimationManager::RequestRegenerationAnimation(this->pos);
	}
	if (poisoned) {
		this->poisoned = false;
		delete pDeath;
		pDeath = ScoreManager::GetScoreCommand(ScoreManager::SCORE_EVENTS::MushroomKilled);
	}
	this->mushroomSprite.SetFrame(0);
	
	this->hp = cn::MUSHROOM_MAX_HP;
}

void Mushroom::Activate() {
	RegisterCollision<Mushroom>(*this);
	shouldDraw = true;
}

void Mushroom::Deactivate() {
	DeregisterCollision<Mushroom>(*this);
	shouldDraw = false;
}
