#include "MushroomField.h"
#include "MushroomController.h"
#include "SoundManager.h"
#include "GridObject.h"
#include "Mushroom.h"
#include "InvisibleBarrier.h"


//------------------
// CONSTRUCTORS
//------------------

MushroomField::MushroomField() {
	pField = new GridObject **[cn::MUSHROOM_FIELD_COLUMNS];
	for (int i = 0; i < cn::MUSHROOM_FIELD_COLUMNS; i++) {
		pField[i] = new GridObject * [cn::MUSHROOM_FIELD_ROWS];
		for (int j = 0; j < cn::MUSHROOM_FIELD_ROWS; j++) {
			pField[i][j] = nullptr;
		}
	}
	
}



//---------------------
//DESTRUCTORS
//---------------------

MushroomField::~MushroomField() {
	for (int i = 0; i < cn::MUSHROOM_FIELD_COLUMNS; i++) {
		delete[] pField[i];
	}
	delete[] pField;
}

//--------------------------
//   PRIVATE FUNCTIONS
//--------------------------

void MushroomField::generateField() {
	//sets initial row and column so we don't need to create new ones.
	for (int i = 0; i < cn::MUSHROOM_FIELD_COLUMNS; i++) {
		int mushroomsSpawnedThisRow = 0;
		//rows-1 because we don't want shrooms spawning in the bottom row.
		//j = 1 so it doesn't spawn on the top row
		for (int j = 1; j < cn::MUSHROOM_FIELD_ROWS-1; j++) {
			if (rand() < cn::MUSHROOM_SPAWN_CHANCE_IFLIMIT && mushroomsSpawnedThisRow < cn::MUSHROOM_SPAWN_MAX_ROW) {
				mushroomCount++;
				mushroomsSpawnedThisRow++;
				pField[i][j] = MushroomController::requestMushroom(convertGridToPosition(sf::Vector2<unsigned char>(i, j)));
				if (j >= cn::PLAYER_GRID_HEIGHT_LIMIT)
					mushroomCountLowerQuadrant++;
			}
		}
	}

	//this is placing a barrier in the space of the player's spawn.
	this->removeMushroomFromField(sf::Vector2<unsigned char>(cn::PLAYER_SPAWN_HORIZONTAL_GRID, cn::PLAYER_SPAWN_HEIGHT_GRID));
	//Trace::out("Mushrooms in Field: %d\n", this->mushroomCount);
}

void MushroomField::generateAttractorField() {
	generateField();

	//top corners
	this->ReplaceSpaceWithBarrier((cn::MUSHROOM_FIELD_COLUMNS / 2) - 5, 2);
	this->ReplaceSpaceWithBarrier((cn::MUSHROOM_FIELD_COLUMNS / 2) + 5, 2);

	int c_mid = 4;
	int top_y = 2;
	//main block
	for (int i = 0; i < 9; i++) {
		for (int j = 0; j < 9; j++) {
			this->ReplaceSpaceWithBarrier((cn::MUSHROOM_FIELD_COLUMNS / 2) - c_mid + i, j + top_y);
		}
	}
	//credits line
	for (int i = 0; i < 11; i++) {
		this->ReplaceSpaceWithBarrier((cn::MUSHROOM_FIELD_COLUMNS / 2) - 5 + i, 13);
	}
	//bonus line
	for (int i = 0; i < 17; i++) {
		this->ReplaceSpaceWithBarrier((cn::MUSHROOM_FIELD_COLUMNS / 2) - 8 + i, 14);
	}
		
	//Trace::out("Happened.\n");


}

void MushroomField::ReplaceSpaceWithBarrier(unsigned char x, unsigned char y) {
	GridObject* g = this->pField[x][y];
	if (g != nullptr)
		g->MarkForDestroy();
	this->removeMushroomFromField(sf::Vector2<unsigned char>(x,y));
	this->spawnBarrier(x, y);
}

sf::Vector2f MushroomField::convertGridToPosition(sf::Vector2<unsigned char> const vector) {
	return sf::Vector2f(vector.x * cn::GRID_SEGMENT_WIDTH, vector.y * cn::GRID_SEGMENT_HEIGHT);
}

sf::Vector2<unsigned char> MushroomField::convertPositionToGrid(sf::Vector2f const vector) {
	return sf::Vector2<unsigned char>((unsigned char)(vector.x / cn::GRID_SEGMENT_WIDTH), (unsigned char)(vector.y / cn::GRID_SEGMENT_HEIGHT));
}

//------------------------
// RECEIVER FUNCTIONS
// -----------------------

//this is if the position is given in terms of x,y on the board
void MushroomField::removeMushroomFromField(sf::Vector2f const vector)
{
	removeMushroomFromField(convertPositionToGrid(vector));
}

//this is if the position is given in terms of an x,y on the grid
void MushroomField::removeMushroomFromField(sf::Vector2<unsigned char> const vector) {
	this->pField[vector.x][vector.y] = nullptr;
	mushroomCount--;
	if (vector.y >= cn::PLAYER_GRID_HEIGHT_LIMIT)
		mushroomCountLowerQuadrant--;
	//Trace::out("Mushroom Count (%d|l:%d)", mushroomCount, mushroomCountLowerQuadrant);
}

GridObject* MushroomField::getMushroom(sf::Vector2<unsigned char> vector) {
	return getMushroom(vector.x, vector.y);
}

GridObject* MushroomField::getMushroom(unsigned char x, unsigned char y) {
	if (x < 0 || x >= cn::MUSHROOM_FIELD_COLUMNS || y < 0 || y >= cn::MUSHROOM_FIELD_ROWS)
		return nullptr;
	else
		return pField[x][y];
}

void MushroomField::spawnMushroom(unsigned char x, unsigned char y) {
	if (this->pField[x][y] == nullptr && y != 0 && y != cn::MUSHROOM_FIELD_ROWS - 1) {
		this->pField[x][y] = MushroomController::requestMushroom(convertGridToPosition(sf::Vector2<unsigned char>(x, y)));
		if (y >= cn::PLAYER_GRID_HEIGHT_LIMIT)
			mushroomCountLowerQuadrant++;
	}
}

void MushroomField::spawnBarrier(unsigned char x, unsigned char y) {
	if (this->pField[x][y] == nullptr && y != 0 && y != cn::MUSHROOM_FIELD_ROWS - 1) {
		this->pField[x][y] = new InvisibleBarrier();
		this->pField[x][y]->Initialize(sf::Vector2f((float)x*cn::PLAYER_SIZE, (float)y*cn::PLAYER_SIZE));
	}
}


void MushroomField::killAllMushrooms() {
	for (int i = 0; i < cn::MUSHROOM_FIELD_COLUMNS; i++) {
		for (int k = 0; k < cn::MUSHROOM_FIELD_ROWS; k++) {
			if (this->pField[i][k] != nullptr) {
				//honestly, right now I'm not sure whose job this should be, the field or the mushroom.
				this->pField[i][k]->MarkForDestroy();
				this->pField[i][k] = nullptr;
			}
		}
	}
	mushroomCount = 0;
	mushroomCountLowerQuadrant = 0;
}

void MushroomField::newGame() {
	this->killAllMushrooms();
	generateField();
}

void MushroomField::newGameAI() {
	this->killAllMushrooms();
	generateAttractorField();
}

void MushroomField::NewRound() {
	this->removeMushroomFromField(sf::Vector2<unsigned char>(cn::PLAYER_SPAWN_HORIZONTAL_GRID, cn::PLAYER_SPAWN_HEIGHT_GRID));
}


bool MushroomField::isBlocked(char x, char y) {
	if (x >= cn::MUSHROOM_FIELD_COLUMNS || x < 0 || y >= cn::MUSHROOM_FIELD_ROWS || y < 1)
		return true;
	else if (this->getMushroom(x, y) == nullptr)
		return false;
	else {
		GridObject* pDummy = this->getMushroom(x, y);
		return true;
	}
		
}

bool MushroomField::isPoisoned(char x, char y) {
	GridObject* mTemp = this->getMushroom(x, y);
	if (mTemp == nullptr)
		return false;
	else
		return mTemp->isPoisoned();
}

void MushroomField::PoisonRequest(sf::Vector2<char> gPos) {
	if (pField[gPos.x][gPos.y] != nullptr) {
		pField[gPos.x][gPos.y]->poisonThis();
	}
}

void MushroomField::Regenerate() {
	for (int i = 0; i < cn::MUSHROOM_FIELD_COLUMNS; i++) {
		for (int k = 0; k < cn::MUSHROOM_FIELD_ROWS; k++) {
			if (this->pField[i][k] != nullptr) {
				//honestly, right now I'm not sure whose job this should be, the field or the mushroom.
				this->pField[i][k]->HealToFull();
			}
		}
	}
}

void MushroomField::RegenerateColumn(int column) {
	for (int i = 0; i < cn::MUSHROOM_FIELD_ROWS; i++) {
		if (this->pField[column][i] != nullptr) {
			this->pField[column][i]->HealToFull();
		}
	}
}

void MushroomField::ActivateField() {
	for (int i = 0; i < cn::MUSHROOM_FIELD_COLUMNS; i++) {
		for (int k = 0; k < cn::MUSHROOM_FIELD_ROWS; k++) {
			if (this->pField[i][k] != nullptr) {
				//honestly, right now I'm not sure whose job this should be, the field or the mushroom.
				this->pField[i][k]->Activate();
			}
		}
	}
}

void MushroomField::DeactivateField() {
	for (int i = 0; i < cn::MUSHROOM_FIELD_COLUMNS; i++) {
		for (int k = 0; k < cn::MUSHROOM_FIELD_ROWS; k++) {
			if (this->pField[i][k] != nullptr) {
				//honestly, right now I'm not sure whose job this should be, the field or the mushroom.
				this->pField[i][k]->Deactivate();
			}
		}
	}
}

void MushroomField::Test() {
	for (int i = 0; i < cn::MUSHROOM_FIELD_COLUMNS; i++) {
		for (int k = 0; k < cn::MUSHROOM_FIELD_ROWS; k++) {
			this->spawnMushroom(i, k);
		}
	}
}

void MushroomField::HighScoreDelete() {
	for (int i = 0; i < 9; i++) {
		int x = (cn::MUSHROOM_FIELD_COLUMNS / 2) - 4 + i;
		int y = 16;
		GridObject* g = this->pField[x][y];
		if (g != nullptr)
			g->MarkForDestroy();
		this->removeMushroomFromField(sf::Vector2<unsigned char>(x, y));
	}
	for (int i = 0; i < 11; i++) {
		int x = (cn::MUSHROOM_FIELD_COLUMNS / 2) - 5 + i;
		int y = 19;
		GridObject* g = this->pField[x][y];
		if (g != nullptr)
			g->MarkForDestroy();
		this->removeMushroomFromField(sf::Vector2<unsigned char>(x, y));
	}
	for (int i = 0; i < 17; i++) {
		int x = (cn::MUSHROOM_FIELD_COLUMNS / 2) - 9 + i;
		int y = 20;
		GridObject* g = this->pField[x][y];
		if (g != nullptr)
			g->MarkForDestroy();
		this->removeMushroomFromField(sf::Vector2<unsigned char>(x, y));
	}
	for (int i = 0; i < 3; i++) {
		int x = (cn::MUSHROOM_FIELD_COLUMNS / 2) - 1 + i;
		int y = 22;
		GridObject* g = this->pField[x][y];
		if (g != nullptr)
			g->MarkForDestroy();
		this->removeMushroomFromField(sf::Vector2<unsigned char>(x, y));
	}
}