#include "Profile.h"
#include "ScoreDisplay.h"
#include "Ship.h"
#include "Definitions.h"
#include "BulletManager.h"
#include "Bullet.h"
#include "SoundManager.h"


Profile::Profile() {
	pShip = new Ship();
	attachedBullet = nullptr;
	Initialize();
}

Profile::~Profile() {
	if(pShip != nullptr)
		pShip->MarkForDestroy();
	if (attachedBullet != nullptr)
		attachedBullet->MarkForDestroy();
}

void Profile::Initialize() {
	lives_remaining = 3;
	wave_id = 0;
	controller_type = CONTROLLER_TYPE::KEYBOARD;
	score = 0;
	bonus_count = 0;
	if (this->field_id == 1) {
		ScoreDisplay::UpdateScoreP1(score);
		ScoreDisplay::UpdateLivesP1(lives_remaining);
	}
	else {
		ScoreDisplay::UpdateScoreP2(score);
		ScoreDisplay::UpdateLivesP2(lives_remaining);
	}
		
	
}

void Profile::addScore(int val) {
	this->score += val;
	if (this->field_id == 1)
		ScoreDisplay::UpdateScoreP1(score);
	else if (this->field_id == 2)
		ScoreDisplay::UpdateScoreP2(score);
}

void Profile::Terminate() {
	//delete sDisplay;
}

void Profile::LoseOneLife() {
	lives_remaining--;
	if (this->field_id == 1)
		ScoreDisplay::UpdateLivesP1(lives_remaining);
	else
		ScoreDisplay::UpdateLivesP2(lives_remaining);
}

bool Profile::AchievedBonus() {
	if (this->score > cn::BONUS_LIFE_SCORE* (bonus_count + 1)) {
		bonus_count++;
		return true;
	}
	else return false;
}

void Profile::AddOneLife() {
	this->lives_remaining++;
	if (this->field_id == 1)
		ScoreDisplay::UpdateLivesP1(lives_remaining);
	else
		ScoreDisplay::UpdateLivesP2(lives_remaining);
}

sf::Vector2f Profile::GetShipPositon() {
	return this->pShip->getPos();
}

void Profile::Shoot() {
	if (attachedBullet != nullptr) {
		//if we have a bullet, do this.
		SoundManager::PlayBlasterSound();
		attachedBullet->Shoot();
		attachedBullet = BulletManager::RequestBullet();
	}
}

void Profile::FillBulletRequest(Bullet* newBullet) {
	attachedBullet = newBullet;
	attachedBullet->Update();
}

void Profile::EndRound() {
	pShip->EndRound();
	if (attachedBullet != nullptr)
		attachedBullet->MarkForDestroy();
}

void Profile::NewRound() {
	pShip->NewRound();
	attachedBullet = BulletManager::RequestBullet();
}

void Profile::NewGame() {
	this->Initialize();
	pShip->NewGame();
}