#include "ScoreDisplay.h"
#include "Definitions.h"

#include "HighScoreManager.h"
#include "GameManager.h"




ScoreDisplay* ScoreDisplay::instance = nullptr;

ScoreDisplay::ScoreDisplay() {
	initials_it = 0;
	p1Score = SpriteString("0", sf::Vector2f((float)(5 * cn::PLAYER_SIZE), 0), &sFont, SpriteString::DRAW_DIRECTION::BACKWARD);
	p2Score = SpriteString("0", sf::Vector2f((float)(cn::MUSHROOM_FIELD_COLUMNS - 1) * cn::PLAYER_SIZE, 0), &sFont, SpriteString::DRAW_DIRECTION::BACKWARD);
	p1Lives = SpriteString("@@@", sf::Vector2f((float)6 * cn::PLAYER_SIZE, 0), &sFont, SpriteString::DRAW_DIRECTION::FORWARD);
	p2Lives = SpriteString("@@@", sf::Vector2f((float)(cn::MUSHROOM_FIELD_COLUMNS - 8) * cn::PLAYER_SIZE, 0) , &sFont, SpriteString::DRAW_DIRECTION::BACKWARD);
	FooterText = SpriteString("Collin Krueger 2019", sf::Vector2f(0.0f, (float)(cn::MUSHROOM_FIELD_ROWS) * cn::PLAYER_SIZE), &sFont, SpriteString::DRAW_DIRECTION::FORWARD);
	
	HighScoresHeaderText = SpriteString("HIGH SCORES", sf::Vector2f((float)((cn::MUSHROOM_FIELD_COLUMNS / 2) - 5.0f) * cn::PLAYER_SIZE, (2.0f * cn::PLAYER_SIZE)), &sFont, SpriteString::DRAW_DIRECTION::FORWARD);
	CreditsText = SpriteString("Credits 2", sf::Vector2f((float)((cn::MUSHROOM_FIELD_COLUMNS / 2) - 5.0f) * cn::PLAYER_SIZE, 13.0f * cn::PLAYER_SIZE), &sFont, SpriteString::DRAW_DIRECTION::FORWARD);
	BonusText = SpriteString("BONUS EVERY 12000", sf::Vector2f((float)((cn::MUSHROOM_FIELD_COLUMNS / 2) - 8.0f) * cn::PLAYER_SIZE, 14.0f * cn::PLAYER_SIZE), &sFont, SpriteString::DRAW_DIRECTION::FORWARD);

	//get high scores
	this->privUpdateHighScores();
	HSDemoText = SpriteString(std::to_string(HighScoreManager::GetHighScore()), sf::Vector2f((float)(13 * cn::PLAYER_SIZE), 0.0f), &sFont, SpriteString::DRAW_DIRECTION::FORWARD);

	GameOverText = SpriteString("GAME OVER", sf::Vector2f((float)((cn::MUSHROOM_FIELD_COLUMNS / 2) - 4.0f) * cn::PLAYER_SIZE, 16.0f * cn::PLAYER_SIZE), &sFont, SpriteString::DRAW_DIRECTION::FORWARD);
	GreatScoreText = SpriteString("GREAT SCORE", sf::Vector2f((float)((cn::MUSHROOM_FIELD_COLUMNS / 2) - 5.0f) * cn::PLAYER_SIZE, 19.0f * cn::PLAYER_SIZE), &sFont, SpriteString::DRAW_DIRECTION::FORWARD);
	EnterYourInitialsText = SpriteString("ENTER YOUR INITIALS", sf::Vector2f((float)((cn::MUSHROOM_FIELD_COLUMNS / 2) - 9.0f) * cn::PLAYER_SIZE, 20.0f * cn::PLAYER_SIZE), &sFont, SpriteString::DRAW_DIRECTION::FORWARD);
	InitialsText = SpriteString("   ", sf::Vector2f((float)((cn::MUSHROOM_FIELD_COLUMNS / 2) - 1.0f) * cn::PLAYER_SIZE, 22.0f * cn::PLAYER_SIZE), &sFont, SpriteString::DRAW_DIRECTION::FORWARD);
}

void ScoreDisplay::privProcessInitials(sf::Uint32 uint) {
	std::string toEdit = InitialsText.GetString();
	char c = (char)(uint);
	if (c == 8) {
		if (initials_it > 0) {
			initials_it--;
			toEdit.at(initials_it) = ' ';
		}
	}
	else if (c == 13 && initials_it == 3) {
		char inits[4];
		inits[0] = toEdit.at(0);
		inits[1] = toEdit.at(1);
		inits[2] = toEdit.at(2);
		HighScoreManager::AddHighScore(inits);
		this->privUpdateHighScores();
		initials_it = 0;
		GameManager::EndHSLoop();
		this->privResetHighScoreText();
		return;
	}
	else {
		if (c >= 'a' && c <= 'z' && initials_it < 3) {
			toEdit.at(initials_it) = c;
			initials_it++;
		}
	}
	InitialsText.UpdateString(toEdit);
}

void ScoreDisplay::privResetHighScoreText() {
	InitialsText.UpdateString("   ");
}

void ScoreDisplay::privUpdateHighScores() {
	std::list<std::pair<int, char[4]>> highScoreInfo = HighScoreManager::GetHighScoresList();
	sf::Vector2f HSpos = sf::Vector2f((float)((cn::MUSHROOM_FIELD_COLUMNS / 2) + 4.0f) * cn::PLAYER_SIZE, 3.0f * cn::PLAYER_SIZE);
	for (std::list<std::pair<int, char[4]>>::iterator it = highScoreInfo.begin(); it != highScoreInfo.end(); ++it) {
		std::string final_string = std::to_string((*it).first) + " " + (*it).second;
		HighScoresText.push_back(SpriteString(final_string, HSpos, &sFont, SpriteString::DRAW_DIRECTION::BACKWARD));
		HSpos.y += cn::PLAYER_SIZE;
	}
}

void ScoreDisplay::privUpdateScoreP1(int nScore) {
	this->p1Score.UpdateString(std::to_string(nScore));
	if (HighScoreManager::HigherThanHighScore(nScore))
		this->HSDemoText.UpdateString(std::to_string(nScore));
}

void ScoreDisplay::privUpdateScoreP2(int nScore) {
	this->p2Score.UpdateString(std::to_string(nScore));
	if (HighScoreManager::HigherThanHighScore(nScore))
		this->HSDemoText.UpdateString(std::to_string(nScore));
}

void ScoreDisplay::privUpdateLivesP1(int nLives) {
	std::string hld;
	for (int i = 0; i < nLives; i++)
		hld.append("@");
	this->p1Lives.UpdateString(hld);
}

void ScoreDisplay::privUpdateLivesP2(int nLives) {
	std::string hld;
	for (int i = 0; i < nLives; i++)
		hld.append("@");
	this->p2Lives.UpdateString(hld);
}

void ScoreDisplay::privDrawP1() {
	p1Score.Draw();
	p1Lives.Draw();
}

void ScoreDisplay::privDrawP2() {
	p2Score.Draw();
	p2Lives.Draw();
}

void ScoreDisplay::privDrawMiscText() {
	FooterText.Draw();
	HSDemoText.Draw();
}

void ScoreDisplay::privDrawAttractor() {
	HighScoresHeaderText.Draw();
	for (std::list<SpriteString>::iterator it = HighScoresText.begin(); it != HighScoresText.end(); ++it) {
		(*it).Draw();
	}
	CreditsText.Draw();
	BonusText.Draw();
}

void ScoreDisplay::privDrawNewHSMode() {
	GameOverText.Draw();
	GreatScoreText.Draw();
	EnterYourInitialsText.Draw();
	InitialsText.Draw();
}

void ScoreDisplay::privTerminate() {
	delete instance;
}