#include "ScoreManager.h"
#include "ScoreCommand.h"
#include "ScoreCommand_Value.h"
#include "ScoreCommand_Distance.h"
#include "Definitions.h"
#include "ProfileManager.h"
#include "Profile.h"
#include "SoundManager.h"

ScoreManager* ScoreManager::instance = nullptr;

ScoreManager::ScoreManager() : ScorpionDeath(cn::SCORPION_DEATH), MushroomDeath(cn::MUSHROOM_DEATH),
								MushroomPoisonDeath(cn::MUSHROOM_POISON_DEATH), MushroomRegeneration(cn::MUSHROOM_REGENERATION_SCORE),
								SpiderDeath_Near(cn::SPIDER_DEATH_NEAR), SpiderDeath_Med(cn::SPIDER_DEATH_MED),
								SpiderDeath_Far(cn::SPIDER_DEATH_FAR), SpiderDistance_Near(cn::SPIDER_DISTANCE_NEAR),
								SpiderDistance_Med(cn::SPIDER_DISTANCE_MED), SpiderDistance_Far(cn::SPIDER_DISTANCE_FAR),
								FleaDeath(cn::FLEA_DEATH), CentipedeBodyDeath(cn::CENTIPEDE_BODY_DEATH), CentipedeHeadDeath(cn::CENTIPEDE_HEAD_DEATH)
{
}

ScoreManager::~ScoreManager() {

}

void ScoreManager::privTerminate() {
	delete instance;
}

void ScoreManager::privAddScore(int val) {
	ProfileManager::GetCurrentPlayer()->addScore(val);
}

void ScoreManager::privProcessScores() {
	ScoreCommand* c;
	while (!CommandQueue.empty()) {
		c = CommandQueue.front();
		c->Execute();

		CommandQueue.pop();
	}
	if (ProfileManager::CurrentPlayerBonus()) {
		SoundManager::PlayBonusSound();
		ProfileManager::BonusLife();
	}
		
}

ScoreCommand* ScoreManager::privGetScoreCommand(SCORE_EVENTS ev) {
	ScoreCommand* pCmd = nullptr;
	switch (ev) {
	case SCORE_EVENTS::ScorpionKilled:
		pCmd = new ScoreCommand_Value(this->ScorpionDeath);
		break;
	case SCORE_EVENTS::MushroomKilled:
		pCmd = new ScoreCommand_Value(this->MushroomDeath);
		break;
	case SCORE_EVENTS::MushroomPoisonKilled:
		pCmd = new ScoreCommand_Value(this->MushroomPoisonDeath);
		break;
	case SCORE_EVENTS::MushroomRegenerated:
		pCmd = new ScoreCommand_Value(this->MushroomRegeneration);
		break;
	case SCORE_EVENTS::SpiderKilled:
		pCmd = new ScoreCommand_Distance(this->SpiderDistance_Near, this->SpiderDistance_Med,
			this->SpiderDistance_Far, this->SpiderDeath_Near, this->SpiderDeath_Med,
			this->SpiderDeath_Far);
		break;
	case SCORE_EVENTS::FleaKilled:
		pCmd = new ScoreCommand_Value(this->FleaDeath);
		break;
	case SCORE_EVENTS::CentipedeBodyDeath:
		pCmd = new ScoreCommand_Value(this->CentipedeBodyDeath);
		break;
	case SCORE_EVENTS::CentipedeHeadDeath:
		pCmd = new ScoreCommand_Value(this->CentipedeHeadDeath);
		break;
	}
	return pCmd;
}

void ScoreManager::privSendScoreCommand(ScoreCommand* sc) {
	CommandQueue.push(sc);
}