#ifndef SCORE_MANAGER_H
#define SCORE_MANAGER_H

#include <queue>

class ScoreCommand;



class ScoreManager
{
private:
	ScoreManager();
	ScoreManager(const ScoreManager&) = delete;
	ScoreManager& operator=(const ScoreManager&) = delete;
	~ScoreManager();
	static ScoreManager* instance;
	static ScoreManager& getInstance() {
		if (!instance)
			instance = new ScoreManager();
		return *instance;
	}
public:
	enum class SCORE_EVENTS {
		ScorpionKilled,
		MushroomKilled,
		MushroomPoisonKilled,
		MushroomRegenerated,
		SpiderKilled,
		FleaKilled,
		CentipedeBodyDeath,
		CentipedeHeadDeath
	};
public:
	//public static methods
	static void AddScore(int val) { getInstance().privAddScore(val); }
	static void ProcessScores() { getInstance().privProcessScores(); }
	static ScoreCommand* GetScoreCommand(SCORE_EVENTS ev) { return getInstance().privGetScoreCommand(ev); }
	static void SendScoreCommand(ScoreCommand* c) { getInstance().privSendScoreCommand(c); }
	static void Terminate() { getInstance().privTerminate(); }
private:
	//private non-static methods
	void privAddScore(int);
	void privProcessScores();
	ScoreCommand* privGetScoreCommand(SCORE_EVENTS);
	void privSendScoreCommand(ScoreCommand*);
	void privTerminate();
private:
	//data members
	const int ScorpionDeath;
	const int MushroomDeath;
	const int MushroomPoisonDeath;
	const int MushroomRegeneration;
	const int SpiderDeath_Near;
	const int SpiderDeath_Med;
	const int SpiderDeath_Far;
	const int SpiderDistance_Near;
	const int SpiderDistance_Med;
	const int SpiderDistance_Far;
	const int FleaDeath;
	const int CentipedeBodyDeath;
	const int CentipedeHeadDeath;
	std::queue<ScoreCommand*> CommandQueue;
};


#endif