#include "Scorpion.h"
#include "MushroomFieldManager.h"
#include "ScoreCommand.h"
#include "ScoreManager.h"
#include "SoundManager.h"
#include "AnimationManager.h"

Scorpion::Scorpion() {
	this->Initialize();
	this->bitmap = &ResourceManager::GetTextureBitmap("scorpionSprite");
	this->SetDrawOrder(22);

	this->pDeath = ScoreManager::GetScoreCommand(ScoreManager::SCORE_EVENTS::ScorpionKilled);

	this->SetCollider(this->cSprite,*bitmap);
}

void Scorpion::Initialize() {
	this->hp = 1;
	this->gridY = rand() % cn::SCORPION_RANGE + 1; //+1 because we want to generate a number between 1 and cn::Scorpion_range
	this->untilNextGridSpaceX = 0;
	this->cSprite = AnimatedSprite(ResourceManager::GetTexture("scorpionSprite"), 4, 1, 8, 0, 3, true, true);
	this->cPos.y = (float)(this->gridY * cn::PLAYER_SIZE);
	if (rand() % 2 == 0)
		spawnLeft();
	else
		spawnRight();
	RegisterCollision<Scorpion>(*this);
}



void Scorpion::spawnRight() {
	this->gridX = cn::MUSHROOM_FIELD_COLUMNS-1;
	this->cPos.x = (float)(cn::SCREEN_WIDTH - cn::PLAYER_SIZE);
	this->horizontalVelocity = -cn::SCORPION_SPEED;
}

void Scorpion::spawnLeft() {
	this->gridX = 0;
	this->cPos.x = 0;
	this->cSprite.scale(-1.0f, 1.0f);
	this->horizontalVelocity = cn::SCORPION_SPEED;
}


void Scorpion::Update() {
	this->cPos.x += this->horizontalVelocity;
	this->untilNextGridSpaceX += (char)cn::SCORPION_SPEED;
	if (gridX < 0 || gridX > cn::MUSHROOM_FIELD_COLUMNS-1) {
		this->MarkForDestroy();
	}
	else if (untilNextGridSpaceX > cn::PLAYER_SIZE) {
		this->untilNextGridSpaceX -= cn::PLAYER_SIZE;
		//Trace::out("Next Gridspace (%d)\n", gridX);
		MushroomFieldManager::PoisonMushroomOnCurrentField(sf::Vector2<char>(gridX, gridY));
		//I know this is horribly inefficient, I will pay more mind to it next week.
		this->gridX += (char)(horizontalVelocity / cn::SCORPION_SPEED);
	}
	
	Critter::Update();
}

void Scorpion::Collision(Bullet* other) {
	ScoreManager::SendScoreCommand(this->pDeath);
	SoundManager::PlayKillSound();
	AnimationManager::RequestRegenerationAnimation(this->cPos);
	this->MarkForDestroy();
}

void Scorpion::Destroy() {
	CritterController::catchScorpionDeath(this);
	DeregisterCollision<Scorpion>(*this);
}

Scorpion::~Scorpion() {
	delete pDeath;
}