
#include "SoundManager.h"
#include "SoundMode.h"
#include "SoundMode_Active.h"
#include "SoundMode_Mute.h"
#include "CritterController.h"
#include "CentipedeManager.h"

SoundManager::SoundManager() {
	this->pMuteMode = new SoundMode_Mute();
	this->pActiveMode = new SoundMode_Active();
	pMode = pMuteMode;
	this->muted = false;
	InitializeSounds();
}

SoundManager* SoundManager::instance = nullptr;

void SoundManager::privMute() {
	pMode = pMuteMode;
	this->StopAllSounds();
	this->muted = true;
}

void SoundManager::privUnMute() {
	pMode = pActiveMode;
	this->muted = false;
	//write code to play sounds that would exist.
	if (CritterController::ExistsScorpion())
		this->privPlayScorpionSound();
	if (CritterController::ExistsSpider())
		this->privPlaySpiderSound();
	if (CentipedeManager::DoesCentipedeExist())
		this->privPlayCentipedeSound();

}

void SoundManager::privTerminate() {
	delete instance;
}

SoundManager::~SoundManager() {
	delete pMuteMode;
	delete pActiveMode;
}

void SoundManager::RequestSound(sf::Sound& snd) {
	this->pMode->Play(snd);
}

void SoundManager::privMuteToggle() {
	if (muted)
		this->privUnMute();
	else
		this->privMute();
}


void SoundManager::privStopAllSounds() {
	this->CentipedeSound.stop();
	this->FleaSound.stop();
	this->SpiderSound.stop();
	this->ScorpionSound.stop();
	this->BlasterSound.stop();
	this->DeathSound.stop();
	this->KillSound.stop();
	this->BonusSound.stop();
}

void SoundManager::privChangeVolume(float volume) {
	if (volume <= 0.0f)
		this->privMute();
	this->CentipedeSound.setVolume(volume);
	this->FleaSound.setVolume(volume);
	this->SpiderSound.setVolume(volume);
	this->ScorpionSound.setVolume(volume);
	this->BlasterSound.setVolume(volume);
	this->DeathSound.setVolume(volume);
	this->KillSound.setVolume(volume);
	this->BonusSound.setVolume(volume);

}

void SoundManager::InitializeSounds() {
	this->CentipedeSoundBuffer = ResourceManager::GetSound("centipedeSound");
	this->FleaSoundBuffer	   = ResourceManager::GetSound("fleaSound");
	this->SpiderSoundBuffer	   = ResourceManager::GetSound("spiderSound");
	this->ScorpionSoundBuffer  = ResourceManager::GetSound("scorpionSound");
	this->BlasterSoundBuffer   = ResourceManager::GetSound("blasterSound");
	this->DeathSoundBuffer	   = ResourceManager::GetSound("deathSound");
	this->KillSoundBuffer	   = ResourceManager::GetSound("killSound");
	this->BonusSoundBuffer	   = ResourceManager::GetSound("bonusSound");

	this->CentipedeSound.setBuffer(this->CentipedeSoundBuffer);
	this->FleaSound.setBuffer(this->FleaSoundBuffer);
	this->SpiderSound.setBuffer(this->SpiderSoundBuffer);
	this->ScorpionSound.setBuffer(this->ScorpionSoundBuffer);
	this->BlasterSound.setBuffer(this->BlasterSoundBuffer);
	this->DeathSound.setBuffer(this->DeathSoundBuffer);
	this->KillSound.setBuffer(this->KillSoundBuffer);
	this->BonusSound.setBuffer(this->BonusSoundBuffer);

	this->CentipedeSound.setLoop(true);
	this->SpiderSound.setLoop(true);
	this->ScorpionSound.setLoop(true);

	

}