#ifndef SOUND_MANAGER_H
#define SOUND_MANAGER_H

#include "Game Components/TEAL/CommonElements.h"

class SoundMode;

class SoundManager
{
private:
	SoundManager();
	SoundManager(const SoundManager&) = default;
	SoundManager& operator=(const SoundManager&) = default;
	~SoundManager();
	static SoundManager* instance;
	static SoundManager& getInstance() {
		if (!instance)
			instance = new SoundManager();
		return *instance;
	}
public:
	//public static functions
	static void MuteToggle() { getInstance().privMuteToggle(); }
	static void Mute() { getInstance().privMute(); }
	static void UnMute() { getInstance().privUnMute(); }
	static void Terminate() { getInstance().privTerminate(); }
	static void PlayCentipedeSound() { getInstance().privPlayCentipedeSound(); }
	static void PlayBlasterSound() { getInstance().privPlayBlasterSound(); }
	static void PlaySpiderSound() { getInstance().privPlaySpiderSound(); }
	static void PlayFleaSound() { getInstance().privPlayFleaSound(); }
	static void PlayScorpionSound() { getInstance().privPlayScorpionSound(); }
	static void PlayDeathSound() { getInstance().privPlayDeathSound(); }
	static void PlayRegenerateSound() { getInstance().privPlayRegenerateSound(); }
	static void PlayKillSound() { getInstance().privPlayKillSound(); }
	static void PlayBonusSound() { getInstance().privPlayBonusSound(); }
	//stops
	static void StopSpiderSound() { getInstance().privStopSpiderSound(); }
	static void StopScorpionSound() { getInstance().privStopScorpionSound(); }
	static void StopCentipedeSound() { getInstance().privStopCentipedeSound(); }
	static void StopAllSounds() { getInstance().privStopAllSounds(); }
	//misc
	static void ChangeVolume(float vol) { getInstance().privChangeVolume(vol); }
	
private:
	//private non-static functions
	void privMuteToggle();
	void privMute();
	void privUnMute();
	void privTerminate();
	void privPlayCentipedeSound() { RequestSound(CentipedeSound); }
	void privPlayBlasterSound() { RequestSound(BlasterSound); }
	void privPlaySpiderSound() { RequestSound(SpiderSound); }
	void privPlayFleaSound() { RequestSound(FleaSound); }
	void privPlayScorpionSound() { RequestSound(ScorpionSound); }
	void privPlayDeathSound() { RequestSound(DeathSound); }
	void privPlayRegenerateSound() { RequestSound(KillSound); }
	void privPlayKillSound() { RequestSound(KillSound); }
	void privPlayBonusSound() { RequestSound(BonusSound); }
	void privStopAllSounds();

	void privStopSpiderSound() { SpiderSound.stop(); }
	void privStopScorpionSound() { ScorpionSound.stop(); }
	void privStopCentipedeSound() { CentipedeSound.stop(); }

	void RequestSound(sf::Sound&);
	void InitializeSounds();

	void privChangeVolume(float);
private:
	//member variables
	//buffers
	sf::SoundBuffer CentipedeSoundBuffer;
	sf::SoundBuffer FleaSoundBuffer;
	sf::SoundBuffer SpiderSoundBuffer;
	sf::SoundBuffer ScorpionSoundBuffer;
	sf::SoundBuffer BlasterSoundBuffer;
	sf::SoundBuffer DeathSoundBuffer;
	sf::SoundBuffer KillSoundBuffer;
	sf::SoundBuffer BonusSoundBuffer;

	sf::Sound CentipedeSound;
	sf::Sound FleaSound;
	sf::Sound SpiderSound;
	sf::Sound ScorpionSound;
	sf::Sound BlasterSound;
	sf::Sound DeathSound;
	sf::Sound KillSound;
	sf::Sound BonusSound;

private:
	SoundMode* pMode;
	bool muted;
	SoundMode* pMuteMode;
	SoundMode* pActiveMode;
};

#endif