#include "Spider.h"
#include "WaveManager.h"
#include "Wave.h"
#include "ScoreCommand.h"
#include "ScoreCommand_Distance.h"
#include "ScoreManager.h"
#include "SoundManager.h"
#include "AnimationManager.h"
#include "ProfileManager.h"
#include "Profile.h"

Spider::Spider() {
	this->bitmap = &ResourceManager::GetTextureBitmap("spiderSprite");
	this->cSprite = AnimatedSprite(ResourceManager::GetTexture("spiderSprite"), 4, 2, 8, 0, 7, true, true);
	this->cPos.x = 0;
	this->cPos.y = 0;
	this->gridX = 0;
	this->gridY = 0;
	//doubles size of the spider
	this->cSprite.setScale(2.0f, 2.0f);
	this->Initialize();

	this->pDeath = ScoreManager::GetScoreCommand(ScoreManager::SCORE_EVENTS::SpiderKilled);

	SetCollider(this->cSprite, *bitmap);
}

void Spider::Update() {
	this->moveHorizontal();
	this->moveVertical();
	if (this->cPos.x < -cn::SPIDER_SIZE || this->cPos.x > cn::SCREEN_WIDTH) {
		this->MarkForDestroy();
	}
	

	Critter::Update();
}

void Spider::Destroy() {
	CritterController::catchSpiderDeath(this);
	
	this->DeregisterCollision<Spider>(*this);
}

void Spider::spawnRight() {
	this->horizontalVelocity = -1*WaveManager::GetCurrentWave()->getSpiderSpeed()*cn::SPIDER_HORIZONTAL_SPEED;
	this->cPos.x = (float)cn::SCREEN_WIDTH;
}

void Spider::spawnLeft() {
	this->horizontalVelocity = WaveManager::GetCurrentWave()->getSpiderSpeed()*cn::SPIDER_HORIZONTAL_SPEED;
	this->cPos.x = 0;
}

void Spider::moveHorizontal() {
	this->cPos.x += this->horizontalVelocity*canMove;
}

void Spider::moveVertical() {
	this->cPos.y += this->verticalVelocity;
	if (cPos.y == cn::SCREEN_HEIGHT-cn::SPIDER_SIZE || cPos.y == cn::SPIDER_MAX_SPAWN_HEIGHT-cn::SPIDER_SIZE) {
		verticalVelocity *= -1;
		randomMoveCheck();
	}
	else if (cPos.y == cn::SPIDER_MAX_SPAWN_HEIGHT + cn::SPIDER_SIZE*((cn::SPIDER_ROWS/2)-1)) {
		if (rand() % 2 == 0) {
			//Trace::out("other way");
			this->verticalVelocity *= -1;
		}
		randomMoveCheck();
	}
}

void Spider::randomMoveCheck() {
	if (rand() > cn::SPIDER_MOVE_CHANCE_IFLIMIT) {
		//need to move horizontal until next breakpoint is reached.
		this->canMove = 0.0f;
	}
	else {
		this->canMove = 1.0f;
	}
}

//future proofing for object pooling on TO-DO
void Spider::Initialize() {
	this->hp = 1;
	this->RegisterCollision<Spider>(*this);
	//temporary
	this->cPos.y = (float)(cn::SCREEN_HEIGHT - ((rand() % (cn::SPIDER_ROWS-2)+2)*cn::SPIDER_SIZE));
	this->verticalVelocity = WaveManager::GetCurrentWave()->getSpiderSpeed() * cn::SPIDER_VERTICAL_SPEED;
	this->canMove = 1.0f;
	//Trace::out("Current Height: %f, Height Until Bounce: %f\n", this->cPos.y, this->distanceToBounce);
	if (rand() % 2 == 0)
		this->spawnLeft();
	else
		this->spawnRight();
}

void Spider::Collision(Bullet *other) {
	sf::Vector2f playerPos = ProfileManager::GetCurrentPlayer()->GetShipPositon();
	int distance = (int)(fabsf(sqrtf(powf(this->cPos.x - playerPos.x, 2) + powf(this->cPos.y - playerPos.y, 2))));
	AnimationManager::RequestSpiderDeathAnimation(this->cPos, distance);
	((ScoreCommand_Distance*)(this->pDeath))->GetDistances(distance);
	ScoreManager::SendScoreCommand(this->pDeath);
	SoundManager::PlayKillSound();
	AnimationManager::RequestRegenerationAnimation(this->cPos);
	MarkForDestroy();
}

Spider::~Spider() {
	delete pDeath;
}