#include "SpriteString.h"
#include "Glyph.h"
#include "GlyphSheet.h"
#include "Definitions.h"

SpriteString::SpriteString(std::string txt, sf::Vector2f nPos, GlyphSheet* font, SpriteString::DRAW_DIRECTION dir) {
	this->stringText = txt;
	this->init_pos = nPos;
	this->stringFont = font;
	this->stringDirection = dir;

	this->GenerateSymbolList();
}

void SpriteString::GenerateSymbolList() {
	stringSymbols.clear();
	sf::Vector2f nextPos = init_pos;
	if (this->stringDirection == SpriteString::DRAW_DIRECTION::BACKWARD) {
		nextPos.x -= (this->stringText.length()-1) * cn::PLAYER_SIZE;
	}
	for (size_t i = 0; i < this->stringText.length(); i++) {
		this->stringSymbols.push_back(this->stringFont->GetGlyph(this->stringText.at(i), nextPos));
			nextPos.x += cn::PLAYER_SIZE;
	}
}

void SpriteString::Draw() {
	for (std::list<Glyph>::iterator it = this->stringSymbols.begin(); it != this->stringSymbols.end(); ++it) {
		it->Draw();
	}
}

void SpriteString::UpdateString(std::string newString) {
	this->stringText = newString;
	GenerateSymbolList();
}

void SpriteString::UpdateDirection(SpriteString::DRAW_DIRECTION ndir) {
	this->stringDirection = ndir;
	GenerateSymbolList();
}

void SpriteString::UpdatePosition(sf::Vector2f npos) {
	this->init_pos = npos;
	GenerateSymbolList();
}