#ifndef WAVE_H
#define WAVE_H

#include "Game Components/TEAL/CommonElements.h"
#include <cmath>
#include "Definitions.h"

class Wave
{
public:
	Wave();
	Wave(const Wave&) = default;
	Wave& operator=(const Wave&) = default;
	~Wave() = default;

	friend class WaveManager;
private:
	void printParameters();
private:
	//Centipede Related Settings
	int centipede_length;
	int centipede_speed;
	int solo_head_count;
	int solo_head_speed;
	//Flea Settings
	bool can_spawn_fleas;
	int flea_trigger_value;
	//Scorpion Settings
	bool can_spawn_scorpions;
	float scorpion_spawn_frequency;
	//Spider Settings
	bool can_spawn_spider;
	float spider_spawn_frequency;
	int spider_speed;

public:
	//public read-only accessors
	bool canSpawnSpider() { return can_spawn_spider; }
	bool canSpawnFlea() { return can_spawn_fleas; }
	bool canSpawnScorpion() { return can_spawn_scorpions; }
	int getCentipedeLength() { return centipede_length; }
	int getCentipedeSpeed() { return centipede_speed; }
	int getSoloHeadCount() { return solo_head_count; }
	int getSoloHeadSpeed() { return solo_head_speed; }
	int getFleaTriggerValue() { return flea_trigger_value; }
	float getScorpionSpawnChance() { return scorpion_spawn_frequency; }
	float getSpiderSpawnChance() { return spider_spawn_frequency; }
	int getSpiderSpeed() { return spider_speed; }
	int getFramesPerUpdate() { 
		return (int)pow(2, cn::SPEED_COUNT-centipede_speed-1);
	}
	int getFramesPerUpdateHead() {
		return (int)pow(2, cn::SPEED_COUNT - solo_head_speed-1); 
	}

};

#endif