#include "WaveManager.h"
#include "Wave.h"

#include <string>

WaveManager::WaveManager() {
	privInitializeWaveData("wave_settings.txt");
}

WaveManager::~WaveManager() {
	while (!waveList.empty()) {
		Wave* pWave = waveList.back();
		waveList.pop_back();
		delete pWave;
	}
}

WaveManager* WaveManager::instance = nullptr;

void WaveManager::privTerminate() {
	delete instance;
}

void WaveManager::privInitializeWaveData(char* fileName) {
	std::string line;
	std::ifstream myFile(fileName);
	//sf::Vector2<std::string> inputVector;

	std::vector<Wave*>::iterator at;
	if (myFile.is_open()) {
		while (getline(myFile, line), line.c_str()[0] != '#');
		waveList.push_back(new Wave);
		at = waveList.end() - 1;
		while (getline(myFile, line)) {
			if (line.c_str()[0] == '#') {
				waveList.push_back(new Wave);
				at = waveList.end() - 1;
			}
			else {
				size_t delim_pos = line.find_first_of(':');
				this->setParameter(*at, line.substr(0, delim_pos).c_str(), line.substr(delim_pos + 1).c_str());
				//Trace::out("%s ||| %s\n", line.substr(0, delim_pos).c_str(), line.substr(delim_pos + 1).c_str());
			}
		}
		myFile.close();
	}
	else {
		Trace::out("Unable to open file.\n");
	}
	waveIterator = waveList.begin();
	//this->privGetCurrentWave()->printParameters();
}

void WaveManager::setParameter(Wave* wave_inst, const char* field, const char* u_Val) {
	switch (this->hash(field)) {
		case INPUT_CODES::CENTIPEDE_LENGTH:
			wave_inst->centipede_length = std::atoi(u_Val);
			break;
		case INPUT_CODES::CENTIPEDE_SPEED:
			wave_inst->centipede_speed = std::atoi(u_Val);
			break;
		case INPUT_CODES::SOLO_HEAD_COUNT:
			wave_inst->solo_head_count = std::atoi(u_Val);
			break;
		case INPUT_CODES::SOLO_HEAD_SPEED:
			wave_inst->solo_head_speed = std::atoi(u_Val);
			break;
		case INPUT_CODES::CAN_SPAWN_FLEAS:
			wave_inst->can_spawn_fleas = std::atoi(u_Val);
			break;
		case INPUT_CODES::FLEA_TRIGGER_VALUE:
			wave_inst->flea_trigger_value= std::atoi(u_Val);
			break;
		case INPUT_CODES::CAN_SPAWN_SCORPIONS:
			wave_inst->can_spawn_scorpions = std::atoi(u_Val);
			break;
		case INPUT_CODES::SCORPION_SPAWN_FREQUENCY:
			wave_inst->scorpion_spawn_frequency= (float)std::atof(u_Val);
			break;
		case INPUT_CODES::CAN_SPAWN_SPIDER:
			wave_inst->can_spawn_spider = std::atoi(u_Val);
			break;
		case INPUT_CODES::SPIDER_SPAWN_FREQUENCY:
			wave_inst->spider_spawn_frequency = (float)std::atof(u_Val);
			break;
		case INPUT_CODES::SPIDER_SPEED:
			wave_inst->spider_speed = std::atoi(u_Val);
			break;
		default:
			Trace::out("Not a valid line: %s%s\n", field, u_Val);
	}
}


INPUT_CODES WaveManager::hash(std::string const& inString) {
	if (inString == "centipede_length") return INPUT_CODES::CENTIPEDE_LENGTH;
	if (inString == "centipede_speed")	return INPUT_CODES::CENTIPEDE_SPEED;
	if (inString == "solo_head_count") return INPUT_CODES::SOLO_HEAD_COUNT;
	if (inString == "solo_head_speed") return INPUT_CODES::SOLO_HEAD_SPEED;
	if (inString == "can_spawn_fleas") return INPUT_CODES::CAN_SPAWN_FLEAS;
	if (inString == "flea_trigger_value") return INPUT_CODES::FLEA_TRIGGER_VALUE;
	if (inString == "can_spawn_scorpions") return INPUT_CODES::CAN_SPAWN_SCORPIONS;
	if (inString == "scorpion_spawn_frequency") return INPUT_CODES::SCORPION_SPAWN_FREQUENCY;
	if (inString == "can_spawn_spider") return INPUT_CODES::CAN_SPAWN_SPIDER;
	if (inString == "spider_spawn_frequency") return INPUT_CODES::SPIDER_SPAWN_FREQUENCY;
	if (inString == "spider_speed") return INPUT_CODES::SPIDER_SPEED;
	return INPUT_CODES::INVALID;
}


void WaveManager::privNextWave() {
	if (waveIterator != waveList.end()-1) {
		++waveIterator;
	}
	else {
		Trace::out("This is the last wave.\n");
	}

}

void WaveManager::privSetWave(size_t wave_id) {
	waveIterator = waveList.begin();
	for (size_t i = 0; i < wave_id; i++) {
		privNextWave();
	}


}