#ifndef WAVE_MANAGER_H
#define WAVE_MANAGER_H

#include "Game Components/TEAL/CommonElements.h"
#include <iostream>
#include <fstream>
#include <vector>

class Wave;

enum INPUT_CODES : unsigned char {
	CENTIPEDE_LENGTH,
	CENTIPEDE_SPEED,
	SOLO_HEAD_COUNT,
	SOLO_HEAD_SPEED,
	CAN_SPAWN_FLEAS,
	FLEA_TRIGGER_VALUE,
	CAN_SPAWN_SCORPIONS,
	SCORPION_SPAWN_FREQUENCY,
	CAN_SPAWN_SPIDER,
	SPIDER_SPAWN_FREQUENCY,
	SPIDER_SPEED,
	INVALID
};


class WaveManager
{
private:
	WaveManager();
	WaveManager(const WaveManager&) = delete;
	WaveManager& operator=(const WaveManager&) = delete;
	~WaveManager();
	static WaveManager* instance;
	static WaveManager& getInstance() {
		if (!instance)
			instance = new WaveManager();
		return *instance;
	}
private:
	//private non-static functions
	void privTerminate();
	void privInitializeWaveData(char*);
	void setParameter(Wave*,const char*, const char*);
	INPUT_CODES hash(std::string const&);
	Wave* privGetCurrentWave() { return *waveIterator; }
	void privNextWave();
	void privSetWave(size_t);
public:
	//public static functions
	static void Terminate() { getInstance().privTerminate(); }
	static void InitializeWaveData(char* fileName) { getInstance().privInitializeWaveData(fileName); }
	static Wave* GetCurrentWave() { return getInstance().privGetCurrentWave(); }
	static void NextWave() { getInstance().privNextWave(); }
	static void SetWave(size_t wave_id) { getInstance().privSetWave(wave_id); }
private:
	//private members
	std::vector<Wave*> waveList;
	std::vector<Wave*>::iterator waveIterator;
};

#endif